#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-01-01 18:00:04
@File: swanlab/data/sdk.py
@IDE: vscode
@Description:
    在此处封装swanlab在日志记录模式下的各种接口
"""
import os
import time
from datetime import datetime
from typing import Union, Dict, Literal, List

import platformdirs

from swanlab.env import SwanLabEnv, create_swanlog_dir, get_swanlog_dir
from swanlab.formatter import (
    check_load_json_yaml,
    check_callback_format,
    check_exp_name_format,
    check_proj_name_format,
    check_desc_format,
    check_tags_format,
    check_run_id_format,
)
from swanlab.log import swanlog
from swanlab.swanlab_settings import Settings, get_settings, set_settings, read_folder_settings
from swanlab.toolkit import SwanKitCallback
from .modules import DataType
from .run import (
    SwanLabRunState,
    SwanLabRun,
    get_run,
    get_metadata,
)
from .store import get_run_store
from .utils import (
    _init_config,
    _load_from_dict,
    _load_from_env,
    _load_list_from_env,
    _create_operator,
    _init_mode,
    should_call_after_init,
    should_call_before_init,
)
from ..core_python import create_client, auth
from ..package import HostFormatter


@should_call_before_init("After calling swanlab.login(), you can't call it again.")
def login(api_key: str = None, host: str = None, web_host: str = None, save: bool = False):
    """
    Login to SwanLab Cloud. If you already have logged in, you can use this function to relogin.
    Every time you call this function, the previous login information will be overwritten.

    [Note that] this function should be called before `init`.

    :param api_key: str, optional
        authentication key, if not provided, the key will be read from the key file.
    :param host: str, optional
        api host, if not provided, the default host will be used.
    :param web_host: str, optional
        web host, if not provided, the default host will be used.
    :param save: bool, optional
        whether to save the api key to the key file, default is False
    :return: LoginInfo
    """
    if SwanLabRun.is_started():
        raise RuntimeError("You must call swanlab.login() before using init()")
    # 检查host是否合法，并格式化，注入到环境变量中
    HostFormatter(host, web_host)()
    # 登录，初始化http对象
    login_info = auth.code_login(api_key, save) if api_key else auth.create_login_info(save)
    create_client(login_info)
    if api_key:
        os.environ[SwanLabEnv.API_KEY.value] = api_key


MODES = Literal["disabled", "cloud", "local", "offline"]


class SwanLabInitializer:
    def __init__(self):
        self.cbs: List[SwanKitCallback] = []

    @should_call_before_init("After calling swanlab.init(), you can't call it again.")
    def register_callbacks(self, callbacks: List[SwanKitCallback]) -> None:
        self.cbs += callbacks

    def init(
        self,
        project: str = None,
        workspace: str = None,
        experiment_name: str = None,
        description: str = None,
        job_type: str = None,
        group: str = None,
        tags: List[str] = None,
        config: Union[dict, str] = None,
        logdir: str = None,
        mode: MODES = None,
        load: str = None,
        public: bool = None,
        callbacks: List[SwanKitCallback] = None,
        settings: Settings = None,
        id: str = None,
        resume: Union[Literal['must', 'allow', 'never'], bool] = None,
        reinit: bool = None,
        **kwargs,
    ) -> SwanLabRun:
        """
        Start a new run to track and log. Once you have called this function, you can use 'swanlab.log' to log data to
        the current run. Meanwhile, you can use 'swanlab.finish' to finish the current run and close the current
        experiment. After calling this function, SwanLab will begin to record the console output of the current process,
        and register a callback function to the exit function.

        Parameters
        ----------
        project : str, optional
            The project name of the current experiment, the default is None,
            which means the current project name is the same as the current working directory.
        workspace : str, optional
            Where the current project is located, it can be an organization or a user (currently only supports yourself).
            The default is None, which means the current entity is the same as the current user.
        experiment_name : str, optional
            The experiment name you currently have open. If this parameter is not provided,
            SwanLab will generate one for you by default.
        description : str, optional
            The experiment description you currently have open,
            used for a more detailed introduction or labeling of the current experiment.
            If you do not provide this parameter, you can modify it later in the web interface.
        job_type: str, optional
            The job type of the current experiment, used to distinguish different types of experiments.
        group : str, optional
            The experiment group of the current experiment, used for grouping experiments.
        tags : List[str], optional
            The tags of the experiment, used for labeling the current experiment.
            If you do not provide this parameter, you can modify it later in the web interface.
        config : Union[dict, str], optional
            If you provide as a dict, it will be used as the configuration of the current experiment.
            If you provide as a string, SwanLab will read the configuration from the file.
            And the configuration file must be in the format of `json` or `yaml`.
            Anyway, you can modify the configuration later after this function is called.
        logdir : str, optional
            The folder will store all the log information generated during the execution of SwanLab.
            If the parameter is None,
            SwanLab will generate a folder named "swanlog" in the same path as the code execution to store the data.
            If you want to visualize the generated log files,
            simply run the command `swanlab watch` in the same path where the code is executed
            (without entering the "swanlog" folder).
            You can also specify your own folder, but you must ensure that the folder exists and preferably does not contain
            anything other than data generated by Swanlab.
            In this case, if you want to view the logs,
            you must use something like `swanlab watch -l ./your_specified_folder` to specify the folder path.
        mode : str, optional
            Allowed values are 'cloud', 'local', 'disabled', 'offline'.
            If the value is 'cloud', data will be uploaded to the cloud and the local log will be saved.
            If the value is 'local', data will only be saved locally and will not be uploaded to the cloud.
            If the value is 'disabled', data will not be saved or uploaded, just parsing the data.
            If the value is 'offline', data will be saved locally without uploading to the cloud.
            BTW, 'online' is an alias for 'cloud'.
        load : str, optional
            If you pass this parameter,SwanLab will search for the configuration file you specified
            (which must be in JSON or YAML format)
            and automatically fill in some explicit parameters of this function for you
            (excluding parameters in `**kwargs` and the parameters if they are None).
            In terms of priority, if the parameters passed to init are `None`,
            SwanLab will attempt to replace them from the configuration file you provided;
            otherwise, it will use the parameters you passed as the definitive ones.
        public : bool, optional
            Whether the project can be seen by anyone, the default is None, which means the project is private.
            Only available in cloud mode while the first time you create the project.
        callbacks : Union[SwanKitCallback, List[SwanKitCallback]], optional
            The callback function that will be triggered when the experiment is finished.
            If you provide a list, all the callback functions in the list will be triggered in order.
        settings: swanlab.swanlab_settings.Settings, optional
            The settings for the current experiment.
        resume : Literal['must', 'allow', 'never'], optional
            Resume the previous run or not:
                - must: You must pass the `id` parameter and the run must exist.
                - allow: If the run exists, it will be resumed, otherwise a new run will be created.
                - never: You cannot pass the `id` parameter, and a new run will be created.
            You can also pass a boolean value, where `True` is equivalent to 'allow' and `False` is equivalent to 'never'.
            [Notice that] This parameter is only valid when mode='cloud'
        id : str, optional
            The run ID of the previous run, which is used to resume the previous run.
        reinit : bool, optional
            Whether to reinitialize SwanLabRun, the default is False.
        """
        # 一个进程同时只能有一个实验在运行
        if SwanLabRun.is_started():
            run = get_run()
            if reinit:
                run.finish()
            else:
                swanlog.warning("You have already initialized a run, the init function will be ignored")
                return run
        # 注册settings
        merge_settings(settings)
        user_settings = get_settings()
        swanlog.level = kwargs.get("log_level", "info")
        # 获取本地文件夹配置，默认从当前工作目录下的swanlog文件夹中读取
        folder_settings = read_folder_settings(get_swanlog_dir())
        # ---------------------------------- 一些变量、格式检查 ----------------------------------
        # 1. 加载参数
        if callbacks is None:
            callbacks = []
        if resume is True:
            resume = 'allow'
        elif resume is False:
            resume = 'never'
        # for https://github.com/SwanHubX/SwanLab/issues/809
        if experiment_name is None and kwargs.get("name", None) is not None:
            experiment_name = kwargs.get("name")
        if description is None and kwargs.get("notes", None) is not None:
            description = kwargs.get("notes")

        # 1.1 从文件中加载数据
        if load:
            load_data = check_load_json_yaml(load, load)
            experiment_name = _load_from_dict(load_data, "experiment_name", experiment_name)
            description = _load_from_dict(load_data, "description", description)
            job_type = _load_from_dict(load_data, "job_type", job_type)
            group = _load_from_dict(load_data, "group", group)
            tags = _load_from_dict(load_data, "tags", tags)
            config = _load_from_dict(load_data, "config", config)
            logdir = _load_from_dict(load_data, "logdir", logdir)
            mode = _load_from_dict(load_data, "mode", mode)
            project = _load_from_dict(load_data, "project", project)
            workspace = _load_from_dict(load_data, "workspace", workspace)
            public = _load_from_dict(load_data, "private", public)
            id = _load_from_dict(load_data, "id", id)
            resume = _load_from_dict(load_data, "resume", resume)
        # 1.2 初始化confi参数
        config = _init_config(config)
        # 如果config是以下几个类别之一，则抛出异常
        if isinstance(config, (int, float, str, bool, list, tuple, set)):
            raise TypeError(
                f"config: {config} (type: {type(config)}) is not a json serialized dict "
                f"(Support type is dict, MutableMapping, omegaconf.DictConfig, Argparse.Namespace and other dict-like objects), "
                "please check it"
            )
        # 1.3 从环境变量中加载参数
        workspace = _load_from_env(SwanLabEnv.WORKSPACE.value, workspace)
        project = _load_from_env(SwanLabEnv.PROJ_NAME.value, project)
        experiment_name = _load_from_env(SwanLabEnv.EXP_NAME.value, experiment_name)
        description = _load_from_env(SwanLabEnv.DESCRIPTION.value, description)
        job_type = _load_from_env(SwanLabEnv.JOB.value, job_type)
        group = _load_from_env(SwanLabEnv.GROUP.value, group)
        tags = _load_list_from_env(SwanLabEnv.TAGS.value, tags)
        resume = _load_from_env(SwanLabEnv.RESUME.value, resume)
        id = _load_from_env(SwanLabEnv.RUN_ID.value, id)
        logdir = _load_from_env(SwanLabEnv.SWANLOG_FOLDER.value, logdir)
        # 2. 部分格式校验
        # 2.1 校验项目名称，默认实验名称为当前目录名
        project = project if project else os.path.basename(os.getcwd())
        p = check_proj_name_format(project)
        if len(p) != len(project):
            swanlog.warning(f"project name is too long, auto cut to {p}")
            project = p
        # 2.2 校验实验名称
        if experiment_name:
            e = check_exp_name_format(experiment_name)
            if experiment_name != e:
                swanlog.warning("The experiment name has been truncated automatically.")
                experiment_name = e
        # 2.3 校验实验描述
        if description:
            d = check_desc_format(description)
            if description != d:
                swanlog.warning("The description has been truncated automatically.")
                description = d
        # 2.4 校验任务类型
        if job_type:
            j = job_type.strip()
            if j != job_type:
                swanlog.warning("The job type has been stripped automatically.")
                job_type = j
        # 2.5 校验实验组
        if group:
            g = group.strip()
            if g != group:
                swanlog.warning("The group has been stripped automatically.")
                group = g
        # 2.6 校验标签
        if tags:
            new_tags = check_tags_format(tags)
            for i in range(len(tags)):
                if tags[i] != new_tags[i]:
                    swanlog.warning("The tag has been truncated automatically.")
                    tags[i] = new_tags[i]
        # 3. 校验回调函数
        callbacks = check_callback_format(self.cbs + callbacks)
        self.cbs = []
        # 5. 校验mode参数并适配 backup 模式
        mode = "cloud" if mode == "online" else mode
        mode, login_info = _init_mode(mode, folder_settings.mode)
        if mode in ["offline", "local"] and user_settings.backup is False:
            raise RuntimeError("You can't use offline or local mode with backup=False!")
        # 6. 校验 resume 与 id
        resume = resume or 'never'
        if resume == 'never':
            # 不允许传递 id
            if id is not None:
                raise RuntimeError("You can't pass id when resume=never or resume=False.")
        elif resume == 'must' or resume == 'allow':
            # 只允许在 cloud 模式下使用 resume
            if mode != "cloud":
                raise RuntimeError("You can only use resume in cloud mode.")
        if resume == "must" and id is None:
            raise ValueError('You must pass id when resume=must.')
        if id is not None:
            check_run_id_format(id)
        run_store = get_run_store()
        # ---------------------------------- 初始化swanlog文件夹 ----------------------------------
        env_key = SwanLabEnv.SWANLOG_FOLDER.value
        if user_settings.backup is False or mode == "disabled":
            # 设置 backup 为 false 时只是给用户一个没有备份的感觉，但是因为 swanlab 架构问题，必须有地方保存
            # 此时我们将日志存在系统运行时目录
            logdir = platformdirs.user_cache_dir(ensure_exists=True, appname="swanlab", appauthor="SwanHubX")
        elif logdir is None:
            # 如果传入了logdir，则将logdir设置为环境变量，代表日志文件存放的路径
            # 如果没有传入logdir，则使用默认的logdir, 即当前工作目录下的swanlog文件夹，但是需要保证目录存在
            logdir = get_swanlog_dir()
        logdir = os.path.abspath(logdir)
        os.environ[env_key] = logdir
        create_swanlog_dir(logdir)
        # ---------------------------------- 设置运行时配置 ----------------------------------
        # 1. 写入运行时配置
        run_store.resume = resume
        run_store.run_id = id
        run_store.project = project
        run_store.workspace = workspace
        run_store.visibility = public
        run_store.job_type = job_type
        run_store.group = group
        run_store.tags = tags
        run_store.description = description
        run_store.run_name = experiment_name
        run_store.swanlog_dir = logdir
        # 2. 启动操作员，注册运行实例
        operator = _create_operator(mode, login_info, callbacks)
        operator.on_init(project, workspace, public=public, logdir=logdir)
        # init结束后应该设置了一些参数
        assert run_store.run_name is not None, "Run name must be set after initialization."
        assert run_store.run_colors is not None, "Run color must be set after initialization."
        assert run_store.run_id is not None, "Run id must be set after initialization."
        assert run_store.new is not None, "Run new status must be set after initialization."
        if run_store.resume == "never":
            assert run_store.new is True, "Run new status must be True when resume never."
        if run_store.new is True:
            # 新实验时一些参数必须为 none
            assert run_store.config is None, "run_store.config should be None when new experiment."
            assert run_store.metrics is None, "run_store.metrics should be None when new experiment."
        else:
            # 恢复实验时一些参数必须存在
            assert isinstance(run_store.config, dict), "run_store.config should be dict when resuming an experiment."
            assert isinstance(run_store.metrics, dict), "run_store.metrics should be dict when resuming an experiment."
        # ---------------------------------- 初始化运行文件夹 ----------------------------------
        run_id = run_store.run_id
        run_dir = None
        while True:
            run_dir is not None and time.sleep(1)
            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
            run_name = "run-{}-{}".format(timestamp, run_id)
            run_dir = os.path.join(logdir, run_name)
            try:
                os.mkdir(run_dir)
                break
            except FileExistsError:
                pass
        assert run_dir is not None, "run_dir should not be None, please check the logdir and run_id"
        run_store.run_dir = run_dir
        assert os.path.exists(run_store.run_dir), f"Run directory {run_store.run_dir} does not exist."
        os.makedirs(run_store.media_dir, exist_ok=True)
        os.makedirs(run_store.log_dir, exist_ok=True)
        os.makedirs(run_store.file_dir, exist_ok=True)
        os.makedirs(run_store.console_dir, exist_ok=True)
        # ---------------------------------- 初始化运行实例 ----------------------------------
        # 系统信息检测
        meta, monitor_funcs = None, None
        # 新实验开启系统信息检测，旧实验暂时不开启
        # 并且只在用户设置开启了元数据收集或硬件监控时才开启
        if run_store.new is True and user_settings.metadata_collect:
            meta, monitor_funcs = get_metadata(run_store.run_dir)
        run = SwanLabRun(run_config=config, operator=operator, metadata=meta, monitor_funcs=monitor_funcs)
        return run


initializer = SwanLabInitializer()

init = initializer.init

register_callbacks = initializer.register_callbacks


@should_call_after_init("You must call swanlab.init() before using log()")
def log(
    data: Dict[str, DataType],
    step: int = None,
    print_to_console: bool = False,
):
    """
    Log a row of data to the current run.
    We recommend that you log data by SwanLabRun.log() method, but you can also use this function to log data.

    Parameters
    ----------
    data : Dict[str, DataType]
        Data must be a dict.
        The key must be a string with 0-9, a-z, A-Z, " ", "_", "-", "/".
        The value must be a `float`, `float convertible object`, `int` or `swanlab.data.BaseType`.
    step : int, optional
        The step number of the current data, if not provided, it will be automatically incremented.
        If step is duplicated, the data will be ignored.
    print_to_console : bool, optional
        Whether to print the data to the console, the default is False.
    """
    run = get_run()
    ll = run.log(data, step)
    print_to_console and print(ll)
    return ll


@should_call_after_init("You must call swanlab.init() before using finish()")
def finish(state: SwanLabRunState = SwanLabRunState.SUCCESS, error=None):
    """
    Finish the current run and close the current experiment
    Normally, swanlab will run this function automatically,
    but you can also execute it manually and mark the experiment as 'completed'.
    Once the experiment is marked as 'completed', no more data can be logged to the experiment by 'swanlab.log'.
    If you mark the experiment as 'CRASHED' manually, `error` must be provided.
    """
    run = get_run()
    if not run.running:
        return swanlog.error("After experiment is finished, you can't call finish() again.")
    run.finish(state, error)


@should_call_before_init("You can't call merge_settings() after swanlab.init()")
def merge_settings(new_settings: Settings):
    """
    合并用户设置到全局设置
    :param new_settings: Settings对象
    :raises TypeError: 当输入不是Settings对象时抛出
    :raises RuntimeError: 当设置已被锁定时抛出
    """
    if new_settings is None:
        return

    if not isinstance(new_settings, Settings):
        raise TypeError("Expected Settings object")

    current_settings = get_settings()
    merged_data = {**current_settings.model_dump(), **new_settings.model_dump()}
    # 更新全局设置
    set_settings(Settings.model_validate(merged_data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/4/3 16:52
@File: conftest.py.py
@IDE: pycharm
@Description:
    配置pytest
"""
import os
import shutil

import pytest

from tutils import TEMP_PATH, reset_some_env, SwanLabEnv


def count_files_in_directory(directory, exclude_prefixes=("__", ".", "~")):
    """
    计算目录下的文件数量
    """
    file_count, folder_count = 0, 0
    for filename in os.listdir(directory):
        file_path = os.path.join(directory, filename)
        if any(filename.startswith(prefix) for prefix in exclude_prefixes):
            continue
        if os.path.isfile(file_path):
            file_count += 1
        elif os.path.isdir(file_path):
            folder_count += 1
            r = count_files_in_directory(file_path)
            file_count += r[0]
            folder_count += r[1]
    return file_count, folder_count


@pytest.fixture(scope="session", autouse=True)
def setup():
    current_dir = os.path.dirname(os.path.abspath(__file__))
    # 记住当前文件夹下的所有文件数量，用于测试结束后比较，确保测试结束后没有多余文件
    pre_file_count, pre_folder_count = count_files_in_directory(current_dir)
    yield
    # 确保测试结束后没有多余文件
    now_file_count, now_folder_count = count_files_in_directory(current_dir)
    assert pre_file_count == now_file_count
    assert pre_folder_count == now_folder_count


@pytest.fixture(scope="function", autouse=True)
def setup_each():
    """
    对每一个测试函数进行设置
    对于每一个测试函数，将环境变量恢复原状，清空temp文件夹后重新创建一个新的
    """
    for key in SwanLabEnv.list():
        if key in os.environ:
            del os.environ[key]
    reset_some_env()
    # 清空temp文件夹
    if os.path.exists(TEMP_PATH):
        shutil.rmtree(TEMP_PATH)
    os.mkdir(TEMP_PATH)
    yield
    import swanlab

    if swanlab.get_run() is not None:
        swanlab.finish()
    from swanlab.data.store import reset_run_store
    from swanlab.data.porter import DataPorter

    DataPorter._reset()  # noqa: _reset 是内部函数，但在测试中需要重置数据导入器

    reset_run_store()



"""
// @author: ComPleHN
// @file: pictorialbar.vue
// @time: 2025/5/28 12:58
// @description: 本文件是对于echarts的 象形柱图 图表测试,本文件有注意事项请看本目录下 README.md
"""
# ---------------------------------------------- Pictorialbar - Pictorialbar_base ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import PictorialBar
from pyecharts.globals import SymbolType

location = ["山西", "四川", "西藏", "北京", "上海", "内蒙古", "云南", "黑龙江", "广东", "福建"]
values = [13, 42, 67, 81, 86, 94, 166, 220, 249, 262]

c1 = (
    PictorialBar()
    .add_xaxis(location)
    .add_yaxis(
        "",
        values,
        label_opts=opts.LabelOpts(is_show=False),
        symbol_size=18,
        symbol_repeat="fixed",
        symbol_offset=[0, 0],
        is_symbol_clip=True,
        symbol=SymbolType.ROUND_RECT,
    )
    .reversal_axis()
    .set_global_opts(
        title_opts=opts.TitleOpts(title="PictorialBar-各省份人口数量（虚假数据）"),
        xaxis_opts=opts.AxisOpts(is_show=False),
        yaxis_opts=opts.AxisOpts(
            axistick_opts=opts.AxisTickOpts(is_show=False),
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(opacity=0)
            ),
        ),
    )
)

# ---------------------------------------------- Pictorialbar - Pictorialbar_multi_custom_symbols ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import PictorialBar

location = ["山西", "四川", "西藏", "北京", "上海", "内蒙古", "云南", "黑龙江", "广东", "福建"]
values = [13, 42, 67, 81, 86, 94, 166, 220, 249, 262]


with open("../assets/echarts/pictorialbar/symbol.json", "r", encoding="utf-8") as f:
    symbols = json.load(f)

c2 = (
    PictorialBar()
    .add_xaxis(["reindeer", "ship", "plane", "train", "car"])
    .add_yaxis(
        "2015",
        [
            {"value": 157, "symbol": symbols["reindeer"]},
            {"value": 21, "symbol": symbols["ship"]},
            {"value": 66, "symbol": symbols["plane"]},
            {"value": 78, "symbol": symbols["train"]},
            {"value": 123, "symbol": symbols["car"]},
        ],
        label_opts=opts.LabelOpts(is_show=False),
        symbol_size=22,
        symbol_repeat="fixed",
        symbol_offset=[0, 5],
        is_symbol_clip=True,
    )
    .add_yaxis(
        "2016",
        [
            {"value": 184, "symbol": symbols["reindeer"]},
            {"value": 29, "symbol": symbols["ship"]},
            {"value": 73, "symbol": symbols["plane"]},
            {"value": 91, "symbol": symbols["train"]},
            {"value": 95, "symbol": symbols["car"]},
        ],
        label_opts=opts.LabelOpts(is_show=False),
        symbol_size=22,
        symbol_repeat="fixed",
        symbol_offset=[0, -25],
        is_symbol_clip=True,
    )
    .reversal_axis()
    .set_global_opts(
        title_opts=opts.TitleOpts(title="PictorialBar-Vehicles in X City"),
        xaxis_opts=opts.AxisOpts(is_show=False),
        yaxis_opts=opts.AxisOpts(
            axistick_opts=opts.AxisTickOpts(is_show=False),
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(opacity=0)
            ),
        ),
    )
)

# ---------------------------------------------- Pictorialbar - Pictorialbar_custom_symbol ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import PictorialBar

location = ["山西", "四川", "西藏", "北京", "上海", "内蒙古", "云南", "黑龙江", "广东", "福建"]
values = [13, 42, 67, 81, 86, 94, 166, 220, 249, 262]


with open("../assets/echarts/pictorialbar/symbol.json", "r", encoding="utf-8") as f:
    symbols = json.load(f)

c3 = (
    PictorialBar()
    .add_xaxis(location)
    .add_yaxis(
        "",
        values,
        label_opts=opts.LabelOpts(is_show=False),
        symbol_size=22,
        symbol_repeat="fixed",
        symbol_offset=[0, -5],
        is_symbol_clip=True,
        symbol=symbols["boy"],
    )
    .reversal_axis()
    .set_global_opts(
        title_opts=opts.TitleOpts(title="PictorialBar-自定义 Symbol"),
        xaxis_opts=opts.AxisOpts(is_show=False),
        yaxis_opts=opts.AxisOpts(
            axistick_opts=opts.AxisTickOpts(is_show=False),
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(opacity=0)
            ),
        ),
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="pictorialbar",
    public=True,
)

swanlab.log(
    {
        "Pictorialbar - Pictorialbar_base": c1,
        "Pictorialbar - Pictorialbar_multi_custom_symbols": c2,
        "Pictorialbar - Pictorialbar_custom_symbol": c3
    }
)



"""
@author: cunyue
@file: log.py
@time: 2025/10/10 22:01
@description: 终端日志模型
"""

from typing import TypedDict

__all__ = ['LogContent']


class LogContent(TypedDict):
    """日志内容字典类型

    结构示例:
    {
        "message": "hello world",
        "create_time": "2025-05-15 18:35:00",
        "epoch": 1
    }
    """

    message: str
    create_time: str
    epoch: int



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-01-01 15:58:58
@File: swanlab/data/run/main.py
@IDE: vscode
@Description:
    在此处定义SwanLabRun类并导出
"""
import os
from typing import Any, Dict, Optional, List, Tuple

from swanlab.data.modules import DataWrapper, FloatConvertible, Line, Echarts, PyEchartsBase, PyEchartsTable
from swanlab.env import get_mode
from swanlab.formatter import check_key_format
from swanlab.log import swanlog
from swanlab.swanlab_settings import reset_settings, get_settings
from swanlab.toolkit import MediaType
from .config import SwanLabConfig
from .exp import SwanLabExp
from .helper import SwanLabRunOperator, RuntimeInfo, SwanLabRunState, MonitorCron
from .metadata import get_requirements, get_conda, HardwareCollector
from .public import SwanLabPublicConfig
from ..store import get_run_store, reset_run_store

MAX_LIST_LENGTH = 108


class SwanLabRun:
    """
    The SwanLabRun class is used for logging during a single experiment.
    There should be only one instance of the SwanLabRun class for each experiment.
    """

    def __init__(
        self,
        metadata: dict = None,
        monitor_funcs: List[HardwareCollector] = None,
        run_config: Any = None,
        operator: SwanLabRunOperator = None,
    ):
        """
        Initializing the SwanLabRun class involves configuring the settings and initiating other logging processes.

        Parameters
        ----------
        run_config : Any, optional
            实验参数配置，可以在web界面中显示，如学习率、batch size等
            不需要做任何限制，但必须是字典类型，可被json序列化，否则会报错
        operator : SwanLabRunOperator, optional
            实验操作员，用于批量处理回调函数的调用，如果不提供此参数(为None)，则会自动生成一个实例
        """
        if self.is_started():
            raise RuntimeError("SwanLabRun has been initialized")
        global run, config
        run_store = get_run_store()
        # ---------------------------------- 初始化类内参数 ----------------------------------
        operator = operator or SwanLabRunOperator()
        # 0. 下面的参数会在实验结束后进行副作用清理
        self.__operator = operator
        self.__state = SwanLabRunState.RUNNING
        self.__monitor_cron: Optional[MonitorCron] = None
        self.__config: Optional[SwanLabConfig] = None
        # 1. 设置常规参数
        self.__mode = get_mode()
        self.__public = SwanLabPublicConfig()
        self.__operator.before_run(None)
        # 2. 初始化配置
        if run_store.config is not None:
            config.update(run_store.config)
        config.update(run_config)
        # FIXME 不要使用setattr来修改私有变量
        setattr(config, "_SwanLabConfig__on_setter", self.__operator.on_runtime_info_update)
        self.__config = config
        # 3. 初始化实验
        self.__run_id = run_store.run_id
        assert self.__run_id is not None, "Run ID must be set before initializing SwanLabRun"
        self.__operator.before_init_experiment(
            run_store.run_id,
            run_store.run_name,
            run_store.description,
            run_store.run_colors,
        )
        run = self
        operator.on_run()
        self.__exp = SwanLabExp(operator=operator)
        # ---------------------------------- 初始化完成 ----------------------------------
        # 执行__save，必须在on_run之后，因为on_run之前部分的信息还没完全初始化
        getattr(config, "_SwanLabConfig__save")()
        # 运行时信息采集
        metadata, requirements, conda = _get_runtime_info(metadata)
        operator.on_runtime_info_update(RuntimeInfo(requirements=requirements, conda=conda, metadata=metadata))
        # 定时采集系统信息
        # 测试时不开启此功能
        # resume时不开启此功能
        if "PYTEST_VERSION" not in os.environ and run_store.resume == 'never':
            if monitor_funcs is not None and len(monitor_funcs) != 0:
                swanlog.debug("Monitor on.")

                # 定义定时任务函数
                def monitor_func():
                    monitor_info_list = [f() for f in monitor_funcs]
                    # 剔除其中为None的数据
                    for monitor_info in monitor_info_list:
                        if monitor_info is None:
                            swanlog.debug("Hardware info is empty. Skip it.")
                            continue
                        for info in monitor_info:
                            key, name, value, cfg = (
                                info['key'],
                                info['name'],
                                info['value'],
                                info['config'],
                            )
                            v = DataWrapper(key, [Line(value)], reference="TIME")
                            self.__exp.add(
                                data=v,
                                key=key,
                                name=name,
                                column_config=cfg,
                                column_class="SYSTEM",
                                section_type="SYSTEM",
                            )

                self.__monitor_cron = MonitorCron(monitor_func)

    def __cleanup(self, error: str = None, interrupt: bool = False):
        """
        停止部分功能，内部清理时调用
        """
        # 1. 停止硬件监控
        if self.__monitor_cron is not None:
            self.__monitor_cron.cancel()
        # 2. 更新状态
        self.__state = SwanLabRunState.SUCCESS if error is None else SwanLabRunState.CRASHED
        # 3. 触发回调
        if get_settings().log_proxy_type not in ['stderr', 'all']:
            error = None
        self.__operator.on_stop(error, interrupt=interrupt)
        # 4. 更新实验 config
        _config = SwanLabConfig(config)
        self.__config = _config
        config.clean()

    def __str__(self) -> str:
        """此类的字符串表示"""
        return "SwanLabRun(run_id={}, mode={}, state={})".format(
            get_run_store().run_id,
            self.__mode,
            self.__state,
        )

    @property
    def id(self) -> Optional[str]:
        """
        The unique identifier for the current run, which is automatically generated by SwanLab.
        User can also set it manually in the `init` function.
        If mode != 'cloud', this will be None.
        """
        if self.mode == "cloud":
            return self.__run_id
        return None

    @property
    def public(self):
        return self.__public

    @property
    def mode(self) -> str:
        return self.__mode

    @property
    def state(self) -> SwanLabRunState:
        return self.__state

    @classmethod
    def get_state(cls) -> SwanLabRunState:
        """
        获取当前实验状态
        """
        global run
        return run.state if run is not None else SwanLabRunState.NOT_STARTED

    @staticmethod
    def is_started() -> bool:
        """
        If the experiment has been initialized, return True, otherwise return False.
        """
        return get_run() is not None

    @property
    def crashed(self) -> bool:
        """
        If the experiment is marked as 'CRASHED', return True, otherwise return False.
        """
        return self.__state == SwanLabRunState.CRASHED

    @property
    def success(self) -> bool:
        """
        If the experiment is marked as 'SUCCESS', return True, otherwise return False.
        """
        return self.__state == SwanLabRunState.SUCCESS

    @property
    def running(self) -> bool:
        """
        If the experiment is marked as 'RUNNING', return True, otherwise return False.
        """
        return self.__state == SwanLabRunState.RUNNING

    @staticmethod
    def finish(state: SwanLabRunState = SwanLabRunState.SUCCESS, error=None, interrupt: bool = False):
        """
        Finish the current run and close the current experiment
        Normally, swanlab will run this function automatically,
        but you can also execute it manually and mark the experiment as 'completed'.
        Once the experiment is marked as 'completed', no more data can be logged to the experiment by 'swanlab.log'.
        After calling this function, you can re-run `swanlab.init` to start a new experiment.

        :param state: The state of the experiment, it can be 'SUCCESS', 'CRASHED' or 'RUNNING'.
        :param error: The error message when the experiment is marked as 'CRASHED'. If not 'CRASHED', it should be None.
        :param interrupt: Whether the experiment is interrupted by the user. If True, experiment will be marked as 'ABORTED'.
        """
        global run, config
        # 分为几步
        # 1. 设置实验状态
        # 2. 清理 run 内部副作用，并触发 on_stop 回调
        # 4. 重置运行时配置 run_store、重置用户设置 settings
        # 5. 返回old_run
        # 上述步骤中只有客户端对象 client 不清空，其余全局变量全部清理
        if run is None:
            raise RuntimeError("The run object is None, please call `swanlab.init` first.")
        if state == SwanLabRunState.CRASHED and error is None:
            raise ValueError("When the state is 'CRASHED', the error message cannot be None.")
        error = error if state == SwanLabRunState.CRASHED else None
        # 清理内部副作用，触发 on_stop 回调
        getattr(run, "_SwanLabRun__cleanup")(error, interrupt=interrupt)
        # 重置输出代理
        swanlog.reset()
        # 清空配置
        reset_run_store()
        reset_settings()
        # 返回旧的 run 对象
        old_run, run = run, None
        return old_run

    @property
    def config(self) -> SwanLabConfig:
        """
        This property allows you to access the 'config' content passed through `init`,
        and allows you to modify it. The latest configuration after each modification
        will be synchronized to the corresponding path by Swanlab. If you have
        enabled the web service, you will notice the changes after refreshing.
        """
        return self.__config

    def log(self, data: dict, step: int = None):
        """
        Log a row of data to the current run. Unlike `swanlab.log`, this api will be called directly based on the
        SwanRun instance, removing the initialization process. Of course, after you call the success/fail method,
        this method will be banned from calling.

        Parameters
        ----------
        data : Dict[str, DataType]
            Data must be a dict.
            The key must be a string with 0-9, a-z, A-Z, " ", "_", "-", "/".
            The value must be a `float`, `float convertible object`, `int`, `swanlab.data.BaseType`, or a nested dict.
            For nested dicts, keys will be joined with dots (e.g., {'a': {'b': 1}} becomes {'a.b': 1}).
        step : int, optional
            The step number of the current data, if not provided, it will be automatically incremented.
            If step is duplicated, the data will be ignored.

        Raises
        ----------
        ValueError:
            Unsupported key names.
        """
        if self.__state != SwanLabRunState.RUNNING:
            raise RuntimeError("After experiment finished, you can no longer log data to the current experiment")
        self.__operator.on_log(data=data, step=step)

        if not isinstance(data, dict):
            return swanlog.error(
                "log data must be a dict, but got {}, SwanLab will ignore records it.".format(type(data))
            )
        # 检查step的类型
        if step is not None and (not isinstance(step, int) or step < 0):
            swanlog.error(
                "'step' must be an integer not less than zero, but got {}, SwanLab will automatically set step".format(
                    step
                )
            )
            step = None

        # 展平嵌套字典
        flattened_data = _flatten_dict(data)

        log_return = {}
        # 遍历data，记录data
        for k, v in flattened_data.items():
            _k = k
            k = check_key_format(k, auto_cut=True)
            if k != _k:
                # 超过255字符，截断
                swanlog.warning(f"Key {_k} is too long, cut to 255 characters.")
                if k in flattened_data.keys():
                    raise ValueError(f'tag: Not supported too long Key "{_k}" and auto cut failed')
            # ---------------------------------- 包装数据 ----------------------------------
            # 输入为可转换为float的数据类型
            if isinstance(v, (int, float, FloatConvertible)):
                v = DataWrapper(k, [Line(v)])
            elif isinstance(v, (PyEchartsBase, PyEchartsTable)):
                v = DataWrapper(k, [Echarts(v)])
            # 为Line类型或者MediaType类型
            elif isinstance(v, (Line, MediaType)):
                v = DataWrapper(k, [v])
            # 为List[MediaType]、List[PyEchartsBase]或者List[Line]类型，且长度大于0，且所有元素类型相同
            elif (
                isinstance(v, list)
                and len(v) > 0
                and all([isinstance(i, (Line, MediaType, PyEchartsBase, PyEchartsTable)) for i in v])
                and all([i.__class__ == v[0].__class__ for i in v])
            ):
                if len(v) > MAX_LIST_LENGTH:
                    swanlog.warning(f"List length '{k}' is too long, cut to {MAX_LIST_LENGTH}.")
                    v = v[:MAX_LIST_LENGTH]
                # echarts 类型需要转换
                if isinstance(v[0], (PyEchartsBase, PyEchartsTable)):
                    v = DataWrapper(k, [Echarts(i) for i in v])
                else:
                    v = DataWrapper(k, v)
            else:
                # 其余情况被当作是非法的数据类型，交给Line处理
                v = DataWrapper(k, [Line(v)])
            # 数据类型的检查将在创建chart配置的时候完成，因为数据类型错误并不会影响实验进行
            metric_info = self.__exp.add(key=k, data=v, step=step)
            log_return[metric_info.column_info.key] = metric_info

        return log_return


def _get_runtime_info(metadata: Optional[dict]) -> Tuple[Optional[dict], Optional[str], Optional[str]]:
    """
    获取当前运行时信息，包括requirements、conda和metadata
    :return: requirements, conda, metadata
    """
    user_settings = get_settings()
    requirements, conda = None, None
    if user_settings.requirements_collect:
        requirements = get_requirements()
    if user_settings.conda_collect:
        conda = get_conda()
    if not user_settings.metadata_collect:
        metadata = None
    return metadata, requirements, conda


def _flatten_dict(d: dict, parent_key='', sep='.') -> dict:
    """Helper method to flatten nested dictionaries with dot notation"""
    items = []
    for k, v in d.items():
        new_key = f"{parent_key}{sep}{k}" if parent_key else k
        if isinstance(v, dict):
            items.extend(_flatten_dict(v, new_key, sep=sep).items())
        else:
            items.append((new_key, v))
    return dict(items)


run: Optional["SwanLabRun"] = None
"""Global runtime instance. After the user calls finish(), run will be set to None."""
config: Optional["SwanLabConfig"] = SwanLabConfig()  # 全局唯一的config对象，不应该重新赋值
"""Global config instance."""


def get_run() -> Optional["SwanLabRun"]:
    """
    Get the current run object. If the experiment has not been initialized, return None.
    """
    global run
    return run


def get_config() -> Optional["SwanLabConfig"]:
    """
    Get the current config object.
    """
    global config
    return config


def get_url() -> Optional["str"]:
    """
    Get the url of the current experiment.
    NOTE: return None if the experiment has not been initialized or mode is not 'cloud'.
    """
    global run
    if run is None:
        return None
    return run.public.cloud.experiment_url


def get_project_url() -> Optional["str"]:
    """
    Get the url of the current project.
    NOTE: return None if the experiment has not been initialized or mode is not 'cloud'.
    """
    global run
    if run is None:
        return None
    return run.public.cloud.project_url



"""
@author: cunyue
@file: log.py
@time: 2025/5/15 18:35
@description: 标准输出、标准错误流拦截代理，支持外界设置/取消回调，基础作用为输出日志
"""

import re
import sys
from typing import List, Tuple, Callable

from swanlab.env import create_time
from swanlab.toolkit import SwanKitLogger
from .counter import AtomicCounter
from .type import LogHandler, LogType, WriteHandler, LogData, ProxyType
from ..toolkit.models.log import LogContent


class SwanLog(SwanKitLogger):
    """
    swanlab 日志类
    继承自 SwanKitLogger 的同时增加标准输出、标准错误留拦截代理功能
    """

    def __init__(self, name=__name__.lower(), level="info"):
        super().__init__(name=name, level=level)
        self.__original_level = level
        # 当前已经代理的输出行数
        self.__counter = AtomicCounter(0)
        # 保存原始的标准输出和标准错误流
        self.__origin_stdout_write = None
        self.__origin_stderr_write = None
        # 代理缓冲区
        self.__stdout_buffer = ""
        self.__stderr_buffer = ""
        # 上传到云端的最大长度
        self.__max_upload_len = None
        # 当前的代理类型
        self.__proxy_type = None

    @property
    def epoch(self):
        return self.__counter.value

    def __create_write_handler(self, write_type: LogType, handler: LogHandler) -> WriteHandler:
        """
        创建一个新的处理器
        """
        origin_write_handler = self.__origin_stdout_write if write_type == 'stdout' else self.__origin_stderr_write

        def get_buffer():
            return self.__stdout_buffer if write_type == 'stdout' else self.__stderr_buffer

        def set_buffer(buffer):
            if write_type == 'stdout':
                self.__stdout_buffer = buffer
            else:
                self.__stderr_buffer = buffer

        max_output_len = self.__max_upload_len

        def write_handler(message: str):
            """
            处理器函数，线程安全
            """
            try:
                origin_write_handler(message)
            except UnicodeEncodeError:
                # 遇到编码问题，直接pass，此时表现为终端不输出
                pass
            except ValueError as e:
                if "I/O operation on closed file" in str(e):
                    # 遇到文件已关闭问题，直接pass，此时表现为终端不输出
                    pass

            # 进行缓冲处理，主要目的是处理进度条输出：
            # 1. 如果 message 不包含换行符，则加入缓冲区，否则跳转步骤2
            # 2. 如果 message 包含换行符，则将换行符之前的内容和当前缓冲区合并，进入步骤3，准备上传
            # 3. 根据换行符分隔为一个个 message，解析 message，如果 message 包含\r
            messages, new_buffer = clean_control_chars(get_buffer() + message)
            set_buffer(new_buffer)
            # 4. 遍历 messages，上传到云端
            if len(messages):
                log_data = LogData(
                    type=write_type,
                    contents=[],
                )
                with self.__counter as counter:
                    for message in messages:
                        log_data['contents'].append(
                            LogContent(
                                message=message[:max_output_len],
                                create_time=create_time(),
                                epoch=counter.increment(),
                            )
                        )
                # 设置回调
                handler(log_data)

        return write_handler

    def __exec_fun_by_type(self, stdout_func: Callable, stderr_func: Callable):
        """
        根据设置的类型执行对应的函数
        :return: None
        """
        if self.__proxy_type == 'all':
            stdout_func()
            stderr_func()
        elif self.__proxy_type == 'stdout':
            stdout_func()
        elif self.__proxy_type == 'stderr':
            stderr_func()
        # 为 none 就不管了

    @property
    def proxied(self):
        """
        判断是否已经开启代理了
        :return: bool
        """
        return self.__origin_stderr_write is not None or self.__origin_stdout_write is not None

    def start_proxy(self, proxy_type: ProxyType, max_log_length: int, handler: LogHandler, epoch: int = None):
        """
        启动代理
        :param max_log_length: 一行日志的最大长度，超过这个长度的日志将被截断，-1 表示不限制
        :param proxy_type: 代理类型，支持 "stdout", "stderr", "all"
        :param handler: 代理处理函数
        :param epoch: 可选参数，设置当前起始 epoch，默认为 None
        """
        if self.proxied:
            raise RuntimeError("Std Proxy is already started")
        # 设置一些状态
        self.__max_upload_len = max_log_length
        self.__proxy_type = proxy_type
        if epoch is not None:
            self.__counter = AtomicCounter(epoch)

        # 设置代理
        def set_stdout():
            self.__stdout_buffer = ""
            self.__origin_stdout_write = sys.stdout.write
            sys.stdout.write = self.__create_write_handler('stdout', handler)

        def set_stderr():
            self.__stderr_buffer = ""
            self.__origin_stderr_write = sys.stderr.write
            sys.stderr.write = self.__create_write_handler('stderr', handler)

        self.__exec_fun_by_type(set_stdout, set_stderr)

    def stop_proxy(self):
        """
        停止代理
        """
        # 如果没有开启代理，则直接返回
        if not self.proxied:
            return

        # 清理标准输出
        def clean_stdout():
            if self.__stdout_buffer:
                sys.stdout.write(self.__stdout_buffer + '\n')
            self.__stdout_buffer = ""
            sys.stdout.write = self.__origin_stdout_write
            self.__origin_stdout_write = None

        # 清理标准错误
        def clean_stderr():
            if self.__stderr_buffer:
                sys.stderr.write(self.__stderr_buffer + '\n')
            self.__stderr_buffer = ""
            sys.stderr.write = self.__origin_stderr_write
            self.__origin_stderr_write = None

        self.__exec_fun_by_type(clean_stdout, clean_stderr)
        self.__counter = AtomicCounter(0)

    def reset(self):
        """
        重置输出流代理和日志等级
        """
        self.stop_proxy()
        self.level = self.__original_level


_ANSI_ESCAPE_RE = re.compile(r'\x1b\[[0-9;]*[a-zA-Z]')  # 匹配ANSI控制码


def clean_control_chars(text) -> Tuple[List[str], str]:
    """
    清理终端控制字符（模拟终端覆盖行为）
    特别，如果当前字符串中不包含换行符，则直接返回，不处理
    如果存在行为空字符串，则将其删除，不在结果中返回
    """
    lines = text.split('\n')
    cleaned_lines = []
    # 最后一行不处理，因为可能是未完成的行
    # 这与当前 “如果当前字符串中不包含换行符，则直接返回，不处理” 的设计是一致的，并且有更多的灵活性: 最后一行将被作为 buffer 缓冲
    for line in lines[:-1]:
        # 移除控制序列
        cleaned_line = remove_control_sequences(line)
        # 使用预编译正则处理字符串，删除ANSI控制码
        cleaned_line = _ANSI_ESCAPE_RE.sub('', cleaned_line)
        cleaned_line and cleaned_lines.append(cleaned_line)
    return cleaned_lines, lines[-1]


def remove_control_sequences(line: str) -> str:
    """
    高效移除控制序列，目前只处理 \r 和 \x1b[A：
    1. \r：回车符，表示光标回到行首
    2. \x1b[A：上移一行，通常用于终端覆盖输出
    我们取最后一个控制序列后的内容
    """
    # 查找最后一个控制序列的位置
    last_cr = line.rfind('\r')
    last_up = line.rfind('\x1b[A')
    # 确定最后出现的位置
    last_control = max(last_cr, last_up)
    if last_control == -1:
        return line  # 没有控制序列，直接返回

    # 根据控制序列类型确定截取位置
    if last_control == last_up:
        return line[last_control + 3 :]  # \x1b[A 长3字符
    else:
        return line[last_control + 1 :]  # \r 长1字符



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/12 21:10
@File: __init__.py
@IDE: pycharm
@Description:
    cli认证模块
"""
from .login import login
from .logout import logout
from .verify import verify



"""
@author: ComPleHN
@file: __init__.py
@time: 2025/5/19 14:01
@desc: 自定义图表，目前集成了 echarts
"""

from typing import Union

import pyecharts
from pyecharts.charts.base import Base

from swanlab.toolkit import MediaBuffer, DataSuite as D, MediaType
from . import echarts
from .table import Table
from .metrics import confusion_matrix, pr_curve, roc_curve

PyEchartsBase = pyecharts.charts.base.Base
"""
pyecharts.charts.base.Base
"""
PyEchartsTable = Table
"""
custom Table inherited from pyecharts.components.table.Table
"""

__all__ = ["echarts", 'Echarts', 'PyEchartsTable', 'PyEchartsBase', "roc_curve", "pr_curve", "confusion_matrix"]


class Echarts(MediaType):
    def __init__(self, chart: Union[PyEchartsBase, PyEchartsTable]):
        super().__init__()
        self._chart = chart
        self.buffer = MediaBuffer()

    # ---------------------------------- 覆写方法 ----------------------------------

    def parse(self):
        # 文件名称
        byte_string = self._chart.dump_options().encode('utf-8')
        hash_name = D.get_hash_by_bytes(byte_string)[:16]

        # 写入buffer
        self.buffer.write(byte_string)  # 写入二进制数据
        self.buffer.seek(0)  # 重置指针到开头，以便后续读取

        filename = f"echarts-step{self.step}-{hash_name}.json"
        return filename, self.buffer

    def get_chart(self):
        return self.Chart.ECHARTS

    def get_section(self):
        return "ECharts"



import mlflow
import random
import swanlab

swanlab.sync_mlflow()

mlflow.set_experiment("mlflow_sync_test")

with mlflow.start_run(run_name="test_run"):
    mlflow.log_param("learning_rate", 0.01)
    mlflow.log_params({"batch_size": 32, "epochs": 10})
    
    for epoch in range(10):
        acc = 1 - 2 ** -epoch - random.random() / epoch
        loss = 2 ** -epoch + random.random() / epoch
        mlflow.log_metric("accuracy", acc, step=epoch)
        mlflow.log_metric("loss", loss, step=epoch)
        
        mlflow.log_metrics({
            "precision": acc * 0.9,
            "recall": acc * 0.8
        }, step=epoch)


"""
// @author: ComPleHN
// @file: table.vue
// @time: 2025/5/29 10:46
// @description: 本文件是对于echarts的 表格组件 图表的测试
"""

# ---------------------------------------------- Table - Table_base ----------------------------------------------
from swanlab import echarts


table = echarts.Table()

headers = ["NO", "Product", "Count"]
rows = [
    [2, "A", 259],
    [3, "B", 123],
    [4, "C", 300],
    [5, "D", 290],
    [6, "E", 1145],
]
table.add(headers, rows)

c1 = table

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="table",
    public=True,
)

swanlab.log({"Table - Table_base": c1})



"""
@author: cunyue
@file: transformers_fake_train.py
@time: 2025/7/18 16:33
@description: 模拟 transformers 训练过程，使用 Qwen2 模型进行简单的训练和评估
NOTE: 该脚本运行于项目根目录，在项目根目录下执行: `python test/integration/transformers/transformers_fake_train.py`
"""

import argparse
import os.path

import dotenv
import torch
from datasets import Dataset
from transformers import Qwen2Config, Qwen2ForCausalLM, TrainingArguments, Trainer

parser = argparse.ArgumentParser(description="Qwen2模型模拟训练脚本")
parser.add_argument("--num_samples", type=int, default=5000, help="生成的样本数量（默认为5000）")
parser.add_argument("--seq_length", type=int, default=32, help="每个序列的长度（默认为32）")
args = parser.parse_args()

dotenv.load_dotenv()

config = Qwen2Config(
    vocab_size=24,  # GPT-2 标准词汇表大小
    hidden_size=16,
    intermediate_size=32,
    num_hidden_layers=3,
    num_attention_heads=4,
    num_key_value_heads=4,
)

model = Qwen2ForCausalLM(config)


seq = torch.randint(0, 24, (args.num_samples, args.seq_length)).tolist()
ds = Dataset.from_dict({"sequence": seq})
ds = ds.train_test_split(0.2)


train_args = TrainingArguments(
    output_dir=os.path.join(os.path.dirname(__file__), "test_transformers_output"),
    max_steps=120,
    per_device_train_batch_size=2,
    report_to="swanlab",
    logging_steps=1,
    eval_strategy="steps",
    eval_steps=0.1,
    remove_unused_columns=False,
)


def collator(examples):
    data = [example["sequence"] for example in examples]
    data = torch.tensor(data, dtype=torch.int)
    return {
        "input_ids": data,
        "attention_mask": torch.ones_like(data),
        "labels": data,
    }


trainer = Trainer(model, train_args, data_collator=collator, train_dataset=ds["train"], eval_dataset=ds["test"])

trainer.train()



"""
@author: cunyue
@file: test_helper.py
@time: 2024/12/6 13:52
@description: 测试工具函数
"""

from swanlab.data.run.helper import check_log_level


def test_check_log_level():
    assert check_log_level(None) == "info"
    assert check_log_level("debug") == "debug"
    assert check_log_level("info") == "info"
    assert check_log_level("warning") == "warning"
    assert check_log_level("error") == "error"
    assert check_log_level("critical") == "critical"
    assert check_log_level("not_exist") == "info"
    assert check_log_level("INFO") == "info"



from tutils import open_dev_mode
import swanlab

swanlab.login(open_dev_mode())

import gymnasium as gym
from stable_baselines3 import PPO
from stable_baselines3.common.monitor import Monitor
from stable_baselines3.common.vec_env import DummyVecEnv
from swanlab.integration.sb3 import SwanLabCallback

config = {
    "policy_type": "MlpPolicy",
    "total_timesteps": 25000,
    "env_name": "CartPole-v1",
}


def make_env():
    env = gym.make(config["env_name"], render_mode="rgb_array")
    env = Monitor(env)
    return env


env = DummyVecEnv([make_env])
model = PPO(
    config["policy_type"],
    env,
    verbose=1,
)

model.learn(
    total_timesteps=config["total_timesteps"],
    callback=SwanLabCallback(
        project="PPO",
        experiment_name="MlpPolicy",
        verbose=2,
    ),
)

swanlab.finish()



"""
@author: cunyue
@file: __init__.py
@time: 2025/6/19 15:09
@description:
"""

from .providers.api_key import *

# NOTE: 未来的其他认证方式应该使用模块的方式导入，比如 from .providers import oauth, api_key 是最基础的认证方式，因此将相关函数全部导出



"""
@author: cunyue
@file: log.py
@time: 2025/6/19 16:22
@description: swanlab 终端日志输出器，添加了一些好看的样式
"""

from typing import Literal, Union

from rich.console import Console
from rich.text import Text

__all__ = ['SwanKitLogger']

Levels = Union[Literal["debug", "info", "warning", "error", "critical"], str]


class SwanKitLogger:

    def __init__(self, name=__name__.lower(), level: Levels = "info", file=None):
        self.console = Console(file=file)
        self.__level: int = 0
        self.__config = {
            "debug": (
                10,
                Text(name, style='grey54 bold', no_wrap=True) + Text(':', style="default"),
            ),
            "info": (
                20,
                Text(name, style='blue bold', no_wrap=True) + Text(':', style="default"),
            ),
            "warning": (
                30,
                Text(name, style='yellow bold', no_wrap=True) + Text(':', style="default"),
            ),
            "error": (
                40,
                Text(name, style='red bold', no_wrap=True) + Text(':', style="default"),
            ),
            "critical": (
                50,
                Text(name, style='red, bold', no_wrap=True) + Text(':', style="default"),
            ),
        }
        self.__can_log = True

        self.level = level

    @property
    def level(self):
        return self.__level

    @level.setter
    def level(self, level: Levels):
        """
        设置日志等级
        :param level: 日志等级，可选值为 debug, info, warning, error, critical，如果传入的值不在可选值中，则默认为 info
        """
        if level not in ("debug", "info", "warning", "error", "critical"):
            _level = 20  # info
        else:
            _level = self.__config[level][0]
        self.__level = _level

    def disable_log(self):
        """
        关闭日志输出，实例化时默认开启
        """
        self.__can_log = False

    def enable_log(self):
        """
        开启日志输出
        """
        self.__can_log = True

    def __print(self, log_level: str, *args, **kwargs):
        """
        打印日志
        """
        if not self.__can_log:
            return
        level, prefix = self.__config[log_level]
        if level < self.__level:
            return
        if kwargs.get("sep") == '':
            prefix += " "
        self.console.print(prefix, *args, **kwargs)

    # 发送调试消息
    def debug(self, *args, **kwargs):
        return self.__print("debug", *args, **kwargs)

    # 发送通知
    def info(self, *args, **kwargs):
        return self.__print("info", *args, **kwargs)

    # 发生警告
    def warning(self, *args, **kwargs):
        return self.__print("warning", *args, **kwargs)

    # 发生错误
    def error(self, *args, **kwargs):
        return self.__print("error", *args, **kwargs)

    # 致命错误
    def critical(self, *args, **kwargs):
        return self.__print("critical", *args, **kwargs)



"""
// @author: ComPleHN
// @file: sunburst.vue
// @time: 2025/5/29 10:39
// @description: 本文件是对于echarts的 旭日图 图表的测试
"""
# ---------------------------------------------- Sunburst - Drink_flavors ----------------------------------------------
from pyecharts.charts import Sunburst
from pyecharts import options as opts

"""
Gallery 使用 pyecharts 1.2.1
参考地址: https://echarts.apache.org/examples/editor.html?c=sunburst-simple

目前无法实现的功能:

1、暂无
"""
data = [
    {
        "name": "Flora",
        "itemStyle": {"color": "#da0d68"},
        "children": [
            {"name": "Black Tea", "value": 1, "itemStyle": {"color": "#975e6d"}},
            {
                "name": "Floral",
                "itemStyle": {"color": "#e0719c"},
                "children": [
                    {
                        "name": "Chamomile",
                        "value": 1,
                        "itemStyle": {"color": "#f99e1c"},
                    },
                    {"name": "Rose", "value": 1, "itemStyle": {"color": "#ef5a78"}},
                    {"name": "Jasmine", "value": 1, "itemStyle": {"color": "#f7f1bd"}},
                ],
            },
        ],
    },
    {
        "name": "Fruity",
        "itemStyle": {"color": "#da1d23"},
        "children": [
            {
                "name": "Berry",
                "itemStyle": {"color": "#dd4c51"},
                "children": [
                    {
                        "name": "Blackberry",
                        "value": 1,
                        "itemStyle": {"color": "#3e0317"},
                    },
                    {
                        "name": "Raspberry",
                        "value": 1,
                        "itemStyle": {"color": "#e62969"},
                    },
                    {
                        "name": "Blueberry",
                        "value": 1,
                        "itemStyle": {"color": "#6569b0"},
                    },
                    {
                        "name": "Strawberry",
                        "value": 1,
                        "itemStyle": {"color": "#ef2d36"},
                    },
                ],
            },
            {
                "name": "Dried Fruit",
                "itemStyle": {"color": "#c94a44"},
                "children": [
                    {"name": "Raisin", "value": 1, "itemStyle": {"color": "#b53b54"}},
                    {"name": "Prune", "value": 1, "itemStyle": {"color": "#a5446f"}},
                ],
            },
            {
                "name": "Other Fruit",
                "itemStyle": {"color": "#dd4c51"},
                "children": [
                    {"name": "Coconut", "value": 1, "itemStyle": {"color": "#f2684b"}},
                    {"name": "Cherry", "value": 1, "itemStyle": {"color": "#e73451"}},
                    {
                        "name": "Pomegranate",
                        "value": 1,
                        "itemStyle": {"color": "#e65656"},
                    },
                    {
                        "name": "Pineapple",
                        "value": 1,
                        "itemStyle": {"color": "#f89a1c"},
                    },
                    {"name": "Grape", "value": 1, "itemStyle": {"color": "#aeb92c"}},
                    {"name": "Apple", "value": 1, "itemStyle": {"color": "#4eb849"}},
                    {"name": "Peach", "value": 1, "itemStyle": {"color": "#f68a5c"}},
                    {"name": "Pear", "value": 1, "itemStyle": {"color": "#baa635"}},
                ],
            },
            {
                "name": "Citrus Fruit",
                "itemStyle": {"color": "#f7a128"},
                "children": [
                    {
                        "name": "Grapefruit",
                        "value": 1,
                        "itemStyle": {"color": "#f26355"},
                    },
                    {"name": "Orange", "value": 1, "itemStyle": {"color": "#e2631e"}},
                    {"name": "Lemon", "value": 1, "itemStyle": {"color": "#fde404"}},
                    {"name": "Lime", "value": 1, "itemStyle": {"color": "#7eb138"}},
                ],
            },
        ],
    },
    {
        "name": "Sour/\nFermented",
        "itemStyle": {"color": "#ebb40f"},
        "children": [
            {
                "name": "Sour",
                "itemStyle": {"color": "#e1c315"},
                "children": [
                    {
                        "name": "Sour Aromatics",
                        "value": 1,
                        "itemStyle": {"color": "#9ea718"},
                    },
                    {
                        "name": "Acetic Acid",
                        "value": 1,
                        "itemStyle": {"color": "#94a76f"},
                    },
                    {
                        "name": "Butyric Acid",
                        "value": 1,
                        "itemStyle": {"color": "#d0b24f"},
                    },
                    {
                        "name": "Isovaleric Acid",
                        "value": 1,
                        "itemStyle": {"color": "#8eb646"},
                    },
                    {
                        "name": "Citric Acid",
                        "value": 1,
                        "itemStyle": {"color": "#faef07"},
                    },
                    {
                        "name": "Malic Acid",
                        "value": 1,
                        "itemStyle": {"color": "#c1ba07"},
                    },
                ],
            },
            {
                "name": "Alcohol/\nFremented",
                "itemStyle": {"color": "#b09733"},
                "children": [
                    {"name": "Winey", "value": 1, "itemStyle": {"color": "#8f1c53"}},
                    {"name": "Whiskey", "value": 1, "itemStyle": {"color": "#b34039"}},
                    {
                        "name": "Fremented",
                        "value": 1,
                        "itemStyle": {"color": "#ba9232"},
                    },
                    {"name": "Overripe", "value": 1, "itemStyle": {"color": "#8b6439"}},
                ],
            },
        ],
    },
    {
        "name": "Green/\nVegetative",
        "itemStyle": {"color": "#187a2f"},
        "children": [
            {"name": "Olive Oil", "value": 1, "itemStyle": {"color": "#a2b029"}},
            {"name": "Raw", "value": 1, "itemStyle": {"color": "#718933"}},
            {
                "name": "Green/\nVegetative",
                "itemStyle": {"color": "#3aa255"},
                "children": [
                    {
                        "name": "Under-ripe",
                        "value": 1,
                        "itemStyle": {"color": "#a2bb2b"},
                    },
                    {"name": "Peapod", "value": 1, "itemStyle": {"color": "#62aa3c"}},
                    {"name": "Fresh", "value": 1, "itemStyle": {"color": "#03a653"}},
                    {
                        "name": "Dark Green",
                        "value": 1,
                        "itemStyle": {"color": "#038549"},
                    },
                    {
                        "name": "Vegetative",
                        "value": 1,
                        "itemStyle": {"color": "#28b44b"},
                    },
                    {"name": "Hay-like", "value": 1, "itemStyle": {"color": "#a3a830"}},
                    {
                        "name": "Herb-like",
                        "value": 1,
                        "itemStyle": {"color": "#7ac141"},
                    },
                ],
            },
            {"name": "Beany", "value": 1, "itemStyle": {"color": "#5e9a80"}},
        ],
    },
    {
        "name": "Other",
        "itemStyle": {"color": "#0aa3b5"},
        "children": [
            {
                "name": "Papery/Musty",
                "itemStyle": {"color": "#9db2b7"},
                "children": [
                    {"name": "Stale", "value": 1, "itemStyle": {"color": "#8b8c90"}},
                    {
                        "name": "Cardboard",
                        "value": 1,
                        "itemStyle": {"color": "#beb276"},
                    },
                    {"name": "Papery", "value": 1, "itemStyle": {"color": "#fefef4"}},
                    {"name": "Woody", "value": 1, "itemStyle": {"color": "#744e03"}},
                    {
                        "name": "Moldy/Damp",
                        "value": 1,
                        "itemStyle": {"color": "#a3a36f"},
                    },
                    {
                        "name": "Musty/Dusty",
                        "value": 1,
                        "itemStyle": {"color": "#c9b583"},
                    },
                    {
                        "name": "Musty/Earthy",
                        "value": 1,
                        "itemStyle": {"color": "#978847"},
                    },
                    {"name": "Animalic", "value": 1, "itemStyle": {"color": "#9d977f"}},
                    {
                        "name": "Meaty Brothy",
                        "value": 1,
                        "itemStyle": {"color": "#cc7b6a"},
                    },
                    {"name": "Phenolic", "value": 1, "itemStyle": {"color": "#db646a"}},
                ],
            },
            {
                "name": "Chemical",
                "itemStyle": {"color": "#76c0cb"},
                "children": [
                    {"name": "Bitter", "value": 1, "itemStyle": {"color": "#80a89d"}},
                    {"name": "Salty", "value": 1, "itemStyle": {"color": "#def2fd"}},
                    {
                        "name": "Medicinal",
                        "value": 1,
                        "itemStyle": {"color": "#7a9bae"},
                    },
                    {
                        "name": "Petroleum",
                        "value": 1,
                        "itemStyle": {"color": "#039fb8"},
                    },
                    {"name": "Skunky", "value": 1, "itemStyle": {"color": "#5e777b"}},
                    {"name": "Rubber", "value": 1, "itemStyle": {"color": "#120c0c"}},
                ],
            },
        ],
    },
    {
        "name": "Roasted",
        "itemStyle": {"color": "#c94930"},
        "children": [
            {"name": "Pipe Tobacco", "value": 1, "itemStyle": {"color": "#caa465"}},
            {"name": "Tobacco", "value": 1, "itemStyle": {"color": "#dfbd7e"}},
            {
                "name": "Burnt",
                "itemStyle": {"color": "#be8663"},
                "children": [
                    {"name": "Acrid", "value": 1, "itemStyle": {"color": "#b9a449"}},
                    {"name": "Ashy", "value": 1, "itemStyle": {"color": "#899893"}},
                    {"name": "Smoky", "value": 1, "itemStyle": {"color": "#a1743b"}},
                    {
                        "name": "Brown, Roast",
                        "value": 1,
                        "itemStyle": {"color": "#894810"},
                    },
                ],
            },
            {
                "name": "Cereal",
                "itemStyle": {"color": "#ddaf61"},
                "children": [
                    {"name": "Grain", "value": 1, "itemStyle": {"color": "#b7906f"}},
                    {"name": "Malt", "value": 1, "itemStyle": {"color": "#eb9d5f"}},
                ],
            },
        ],
    },
    {
        "name": "Spices",
        "itemStyle": {"color": "#ad213e"},
        "children": [
            {"name": "Pungent", "value": 1, "itemStyle": {"color": "#794752"}},
            {"name": "Pepper", "value": 1, "itemStyle": {"color": "#cc3d41"}},
            {
                "name": "Brown Spice",
                "itemStyle": {"color": "#b14d57"},
                "children": [
                    {"name": "Anise", "value": 1, "itemStyle": {"color": "#c78936"}},
                    {"name": "Nutmeg", "value": 1, "itemStyle": {"color": "#8c292c"}},
                    {"name": "Cinnamon", "value": 1, "itemStyle": {"color": "#e5762e"}},
                    {"name": "Clove", "value": 1, "itemStyle": {"color": "#a16c5a"}},
                ],
            },
        ],
    },
    {
        "name": "Nutty/\nCocoa",
        "itemStyle": {"color": "#a87b64"},
        "children": [
            {
                "name": "Nutty",
                "itemStyle": {"color": "#c78869"},
                "children": [
                    {"name": "Peanuts", "value": 1, "itemStyle": {"color": "#d4ad12"}},
                    {"name": "Hazelnut", "value": 1, "itemStyle": {"color": "#9d5433"}},
                    {"name": "Almond", "value": 1, "itemStyle": {"color": "#c89f83"}},
                ],
            },
            {
                "name": "Cocoa",
                "itemStyle": {"color": "#bb764c"},
                "children": [
                    {
                        "name": "Chocolate",
                        "value": 1,
                        "itemStyle": {"color": "#692a19"},
                    },
                    {
                        "name": "Dark Chocolate",
                        "value": 1,
                        "itemStyle": {"color": "#470604"},
                    },
                ],
            },
        ],
    },
    {
        "name": "Sweet",
        "itemStyle": {"color": "#e65832"},
        "children": [
            {
                "name": "Brown Sugar",
                "itemStyle": {"color": "#d45a59"},
                "children": [
                    {"name": "Molasses", "value": 1, "itemStyle": {"color": "#310d0f"}},
                    {
                        "name": "Maple Syrup",
                        "value": 1,
                        "itemStyle": {"color": "#ae341f"},
                    },
                    {
                        "name": "Caramelized",
                        "value": 1,
                        "itemStyle": {"color": "#d78823"},
                    },
                    {"name": "Honey", "value": 1, "itemStyle": {"color": "#da5c1f"}},
                ],
            },
            {"name": "Vanilla", "value": 1, "itemStyle": {"color": "#f89a80"}},
            {"name": "Vanillin", "value": 1, "itemStyle": {"color": "#f37674"}},
            {"name": "Overall Sweet", "value": 1, "itemStyle": {"color": "#e75b68"}},
            {"name": "Sweet Aromatics", "value": 1, "itemStyle": {"color": "#d0545f"}},
        ],
    },
]

c1 = (
    Sunburst(init_opts=opts.InitOpts(width="1000px", height="600px"))
    .add(
        "",
        data_pair=data,
        highlight_policy="ancestor",
        radius=[0, "95%"],
        sort_="null",
        levels=[
            {},
            {
                "r0": "15%",
                "r": "35%",
                "itemStyle": {"borderWidth": 2},
                "label": {"rotate": "tangential"},
            },
            {"r0": "35%", "r": "70%", "label": {"align": "right"}},
            {
                "r0": "70%",
                "r": "72%",
                "label": {"position": "outside", "padding": 3, "silent": False},
                "itemStyle": {"borderWidth": 3},
            },
        ],
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="Sunburst-官方示例"))
    .set_series_opts(label_opts=opts.LabelOpts(formatter="{b}"))
)

# ---------------------------------------------- Sunburst - Basic_sunburst ----------------------------------------------
from pyecharts.charts import Sunburst
from pyecharts import options as opts

"""
Gallery 使用 pyecharts 1.1.0
参考地址: https://echarts.apache.org/examples/editor.html?c=sunburst-simple

目前无法实现的功能:

1、
"""

data = [
    opts.SunburstItem(
        name="Grandpa",
        children=[
            opts.SunburstItem(
                name="Uncle Leo",
                value=15,
                children=[
                    opts.SunburstItem(name="Cousin Jack", value=2),
                    opts.SunburstItem(
                        name="Cousin Mary",
                        value=5,
                        children=[opts.SunburstItem(name="Jackson", value=2)],
                    ),
                    opts.SunburstItem(name="Cousin Ben", value=4),
                ],
            ),
            opts.SunburstItem(
                name="Father",
                value=10,
                children=[
                    opts.SunburstItem(name="Me", value=5),
                    opts.SunburstItem(name="Brother Peter", value=1),
                ],
            ),
        ],
    ),
    opts.SunburstItem(
        name="Nancy",
        children=[
            opts.SunburstItem(
                name="Uncle Nike",
                children=[
                    opts.SunburstItem(name="Cousin Betty", value=1),
                    opts.SunburstItem(name="Cousin Jenny", value=2),
                ],
            )
        ],
    ),
]

c2 = (
    Sunburst(init_opts=opts.InitOpts(width="1000px", height="600px"))
    .add(series_name="", data_pair=data, radius=[0, "90%"])
    .set_global_opts(title_opts=opts.TitleOpts(title="Sunburst-基本示例"))
    .set_series_opts(label_opts=opts.LabelOpts(formatter="{b}"))
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="sunburst",
    public=True,
)

swanlab.log(
    {
        "Sunburst - Drink_flavors": c1,
        "Sunburst - Basic_sunburst": c2
    }
)


"""
@author: cunyue
@file: test_filter.py
@time: 2025/7/19 21:26
@description: 测试过滤工具函数
"""

import pytest

from swanlab.data.porter.utils import filter_epoch, filter_metric, filter_column


@pytest.fixture
def mock_metrics():
    return {"key1": ("mock", "mock", None, 1)}


def test_returns_true_when_key_not_in_metrics(mock_metrics):
    assert filter_column("key2", mock_metrics) is True


def test_returns_false_when_key_in_metrics(mock_metrics):
    assert filter_column("key1", mock_metrics) is False


def test_returns_true_when_key_not_in_metrics_for_metric_filter(mock_metrics):
    assert filter_metric("key2", 2, mock_metrics) is True


def test_returns_true_when_step_greater_than_latest_step(mock_metrics):
    assert filter_metric("key1", 2, mock_metrics) is True


def test_returns_false_when_step_not_greater_than_latest_step(mock_metrics):
    assert filter_metric("key1", 1, mock_metrics) is False


def test_returns_true_when_epoch_greater_than_now_epoch():
    assert filter_epoch(5, 3) is True


def test_returns_false_when_epoch_not_greater_than_now_epoch():
    assert filter_epoch(3, 5) is False


def test_returns_true_when_now_epoch_is_none():
    assert filter_epoch(5, None) is True



"""
@author: cunyue
@file: __init__.py
@time: 2025/6/20 13:30
@description: 历史版本的备份、上传协议采用 JSON 序列化实现
为了保证向下兼容性，在此保留相关模型定义
本质上是增加了一层中间层，此中间层的作用是实现 swanlab.toolkit 中模型与 swanlab.core_python.uploader 中模型的相互转换
"""

import json
import os.path
from typing import Optional, List, Literal, Tuple, Union

import yaml
from pydantic import BaseModel as PydanticBaseModel

from swanlab.core_python.uploader import FileModel, ScalarModel, ColumnModel, LogModel, MediaModel
from swanlab.log.type import LogData
from swanlab.toolkit import (
    ColumnInfo,
    ColumnConfig,
    RuntimeInfo,
    MetricInfo,
    ColumnClass,
    SectionType,
    YRange,
    ChartReference,
    MediaBuffer,
)
from swanlab.toolkit.models.log import LogContent


class BaseModel(PydanticBaseModel):

    def __getitem__(self, key):
        return getattr(self, key)

    def to_record(self) -> str:
        """
        将模型转换为 JSON 字符串
        """
        return (
            json.dumps(
                {
                    "model_type": type(self).__name__,
                    "data": self.model_dump(),
                },
                ensure_ascii=False,
            )
            + "\n"
        )

    @classmethod
    def from_record(cls, data: str) -> Tuple[str, 'BaseModel']:
        """
        从 JSON 字符串创建模型实例
        """
        # 最后一个字符串为换行符，需要删除
        data = json.loads(data[:-1])
        model_type = data["model_type"]
        if model_type not in backup_models:
            raise ValueError(f"Unsupported model type: {model_type}")
        return backup_models[model_type].model_validate(data["data"])


class Header(BaseModel):
    backup_type: Literal["DEFAULT"]
    create_time: str  # 备份文件创建时间


class Footer(BaseModel):
    success: bool
    create_time: str


class Project(BaseModel):
    name: str  # 项目名称
    workspace: Optional[str]  # 工作空间名称
    public: Optional[bool]  # 项目是否公开


class Experiment(BaseModel):
    id: str  # 实验ID
    name: str  # 实验名称
    colors: List[str]  # 实验颜色
    description: Optional[str]  # 实验描述
    tags: Optional[List[str]]  # 实验标签


class Log(BaseModel):
    create_time: str  # 日志创建时间
    message: str  # 日志消息内容
    epoch: Optional[int]  # 日志所属的训练轮次
    level: Literal['INFO', 'WARN', 'ERROR']  # 日志级别

    @classmethod
    def from_log_data(cls, log_data: LogData) -> List['Log']:
        """
        从 LogData 对象创建 Log 实例
        """
        level = 'INFO' if log_data['type'] == 'stdout' else 'WARN'
        l = []
        for item in log_data['contents']:
            l.append(
                cls.model_validate(
                    {
                        "create_time": item["create_time"],
                        "epoch": item.get("epoch"),
                        "message": item["message"],
                        "level": level,
                    }
                )
            )
        return l

    def to_log_model(self) -> LogModel:
        return LogModel(
            level=self.level,
            contents=[LogContent(create_time=self.create_time, epoch=self.epoch, message=self.message)],
        )


class Runtime(BaseModel):
    conda_filename: Optional[str]  # Conda 环境文件名
    requirements_filename: Optional[str]  # Python requirements 文件名
    metadata_filename: Optional[str]  # 系统元数据名
    config_filename: Optional[str]  # 用户自定义配置文件名

    @classmethod
    def from_runtime_info(cls, runtime_info: RuntimeInfo):
        """
        从 RuntimeInfo 对象创建 Runtime 实例
        """
        return cls.model_validate(
            {
                "conda_filename": runtime_info.conda.name if runtime_info.conda else None,
                "requirements_filename": runtime_info.requirements.name if runtime_info.requirements else None,
                "metadata_filename": runtime_info.metadata.name if runtime_info.metadata else None,
                "config_filename": runtime_info.config.name if runtime_info.config else None,
            }
        )

    def to_file_model(self, file_dir) -> FileModel:
        """
        将 Runtime 实例转换为 RuntimeInfo 实例
        """
        conda, requirements, metadata, config = None, None, None, None
        if self.conda_filename:
            conda_path = os.path.join(file_dir, self.conda_filename)
            if not os.path.exists(os.path.join(file_dir, self.conda_filename)):
                raise FileNotFoundError(f"Conda file {self.conda_filename} not found in {file_dir}")
            conda = open(conda_path, "r", encoding="utf-8").read()
        if self.requirements_filename:
            requirements_path = os.path.join(file_dir, self.requirements_filename)
            if not os.path.exists(requirements_path):
                raise FileNotFoundError(f"Requirements file {self.requirements_filename} not found in {file_dir}")
            requirements = open(requirements_path, "r", encoding="utf-8").read()
        if self.metadata_filename:
            metadata_path = os.path.join(file_dir, self.metadata_filename)
            if not os.path.exists(metadata_path):
                raise FileNotFoundError(f"Metadata file {self.metadata_filename} not found in {file_dir}")
            metadata = json.loads(open(metadata_path, "r", encoding="utf-8").read())
        if self.config_filename:
            config_path = os.path.join(file_dir, self.config_filename)
            if not os.path.exists(config_path):
                raise FileNotFoundError(f"Config file {self.config_filename} not found in {file_dir}")
            config = yaml.safe_load(open(config_path, "r", encoding="utf-8").read())

        return FileModel(conda=conda, requirements=requirements, metadata=metadata, config=config)


class Column(BaseModel):

    key: str  # 列的唯一标识符
    kid: str  # 列的唯一标识符（可能是一个ID或其他标识）
    name: Optional[str]  # 列的名称
    cls: ColumnClass  # 列的类型，可能是自定义列或系统生成列
    column_type: str  # 列对应的列类型
    chart_reference: ChartReference  # 列对应图表的参考系，可能是步数或时间
    section_name: Optional[str]  # 列所属的组名
    section_type: SectionType  # 列的组类型，可能是自定义组或系统组
    section_sort: Optional[int]  # 列在组中的排序位置
    error: Optional[dict]  # 列的错误信息，如果有的话

    # ---------------------------------- ColumnConfig ----------------------------------
    y_range: YRange  # y轴范围配置
    chart_name: Optional[str]  # 图表名称
    chart_index: Optional[str]  # 图表索引
    metric_name: Optional[str]  # 指标名称
    metric_color: Optional[Tuple[str, str]]  # 指标颜色

    @classmethod
    def from_column_info(cls, column_info: ColumnInfo):
        """
        从 ColumnInfo 对象创建 Column 实例
        """
        error = None
        if column_info.error is not None:
            error = {"data_class": column_info.error.got, "excepted": column_info.error.expected}
        # 这里有些比较抽象的地方：
        # 云端版会自动处理不同类型的数据放在不同的组中，所以如果key没有设置成 {section}/{name} 之类的样式，不需要传递section的名称
        # 但是本地版不会，本地版依靠swanlab的处理结果指定列，所以在Data类型上必须定义获取section_name的方法
        # 云端版不需要这样做，因为云端版会自动处理
        # 因为云端版的设计更加先进，云端版对“列”（本地版叫namespace）做了不同的类型标注，但是本地版没有这个概念
        # 所以这里需要判断一下，如果列类型不为SYSTEM且不是 {section}/{name} 之类的格式，就不传递section_name
        if column_info.section_type == "PUBLIC":
            section_name = None if "/" not in column_info.key else column_info.section_name
        elif column_info.section_type == "SYSTEM":
            section_name = column_info.section_name
        else:
            section_name = None
        return cls.model_validate(
            {
                "key": column_info.key,
                "kid": column_info.kid,
                "name": column_info.name,
                "cls": column_info.cls,
                "column_type": column_info.chart_type.value.column_type,
                "chart_reference": column_info.chart_reference,
                "section_name": section_name,
                "section_type": column_info.section_type,
                "section_sort": column_info.section_sort,
                "error": error,
                "y_range": column_info.config.y_range if column_info.config else None,
                "chart_name": column_info.config.chart_name if column_info.config else None,
                "chart_index": column_info.config.chart_index if column_info.config else None,
                "metric_name": column_info.config.metric_name if column_info.config else None,
                "metric_color": column_info.config.metric_color if column_info.config else None,
            }
        )

    def to_column_model(self) -> ColumnModel:
        """
        将 Column 实例转换为 ColumnModel 实例
        """
        return ColumnModel(
            key=self.key,
            name=self.name,
            cls=self.cls,
            typ=self.column_type,
            section_name=self.section_name,
            section_type=self.section_type,
            config=ColumnConfig(
                y_range=self.y_range,
                chart_name=self.chart_name,
                chart_index=self.chart_index,
                metric_name=self.metric_name,
                metric_color=self.metric_color,
            ),
            error=self.error,
        )


class Metric(BaseModel):

    @classmethod
    def from_metric_info(cls, metric_info: MetricInfo) -> Union['Scalar', 'Media']:
        """
        指标类型比较特殊，因为上传到后端时分不同接口上传
        此函数根据 MetricInfo 的类型返回对应的 Metric 子类实例
        :param metric_info: MetricInfo 对象
        """
        # 标量类型
        if metric_info.column_info.chart_type == metric_info.column_info.chart_type.LINE:
            return Scalar.model_validate(
                {
                    "metric": metric_info.metric,
                    "key": metric_info.column_info.key,
                    "step": metric_info.metric_step,
                    "epoch": metric_info.metric_epoch,
                }
            )
        buffers_name = []
        if metric_info.metric_buffers is not None:
            for name in metric_info.metric['data']:
                buffers_name.append(name)

        # 媒体类型
        return Media.model_validate(
            {
                "metric": metric_info.metric,
                "key": metric_info.column_info.key,
                "kid": metric_info.column_info.kid,
                "key_encoded": metric_info.column_info.key_encode,
                "step": metric_info.metric_step,
                "epoch": metric_info.metric_epoch,
                "buffers_name": buffers_name if len(buffers_name) else None,
            }
        )


class Scalar(Metric):
    metric: dict  # 标量指标数据，通常是一个字典，包含指标名称和对应的值
    key: str  # 标量指标的唯一标识符
    step: int  # 标量指标的步数
    epoch: int  # 标量指标的训练轮次

    def to_scalar_model(self) -> ScalarModel:
        """
        将 Scalar 实例转换为 ScalarModel 实例
        """
        return ScalarModel(
            metric=self.metric,
            key=self.key,
            step=self.step,
            epoch=self.epoch,
        )


class Media(Metric):
    metric: dict  # 媒体指标数据，通常是一个字典，包含媒体类型和对应的文件路径或URL
    key: str  # 媒体指标的唯一标识符
    kid: int  # 当前实验下，列的唯一id，与保存路径等信息有关，与云端请求无关
    key_encoded: str  # 编码后的键值
    step: int  # 媒体指标的步数
    epoch: int  # 媒体指标的训练轮次
    buffers_name: Optional[List[str]]  # 媒体文件的名称

    def to_media_model(self, media_dir: str) -> MediaModel:
        """
        将 Media 实例转换为 MediaModel 实例
        """
        buffers = []
        # 回复原本的 MediaBuffer 对象
        if self.buffers_name:
            for i, buffer_name in enumerate(self.buffers_name):
                buffer = MediaBuffer()
                buffer.write(open(os.path.join(media_dir, str(self.kid), buffer_name), "rb").read())
                buffer.file_name = "{}/{}".format(self.key_encoded, self.metric["data"][i])
                buffers.append(buffer)

        return MediaModel(
            metric=self.metric,
            key=self.key,
            key_encoded=self.key_encoded,
            step=self.step,
            epoch=self.epoch,
            buffers=buffers if len(buffers) else None,
        )


backup_models = {
    model.__name__: model
    for model in [
        Header,
        Project,
        Experiment,
        Log,
        Runtime,
        Column,
        Scalar,
        Media,
        Footer,
    ]
}



"""
@author: cunyue
@file: proxy_tqdm.py
@time: 2025/5/17 21:23
@description: 测试tqdm的进度条在 swanlab 终端代理下的表现
"""

import time

from tqdm import tqdm

import swanlab

swanlab.init(project='log-tqdm')

# 在循环中显示进度条
for i in tqdm(range(100)):
    time.sleep(0.01)
    if i % 10 == 0:
        swanlab.log({"progress": i}, print_to_console=True)



"""
@author: cunyue
@file: test_cloud.py
@time: 2025/6/26 22:55
@description: 测试云端回调功能
"""

import os

import pytest

import swanlab
import tutils as T
from swanlab.core_python import reset_client, get_client
from swanlab.core_python.auth.providers.api_key import login_by_key
from swanlab.data.callbacker import CloudPyCallback
from swanlab.data.store import get_run_store
from swanlab.env import SwanLabEnv
from tutils.setup import mock_login_info, UseMockRunState


def test_create_http(monkeypatch):
    """
    测试登录信息存在时的行为
    """
    login_info = mock_login_info()
    if os.getenv(SwanLabEnv.API_KEY.value):
        del os.environ[SwanLabEnv.API_KEY.value]
    with UseMockRunState():
        reset_client()
        with pytest.raises(ValueError):
            get_client()

        # 默认情况下会报错，因为没有登录信息
        def raise_error(_):
            raise RuntimeError("Mocked error for testing")

        monkeypatch.setattr("getpass.getpass", raise_error)
        with pytest.raises(RuntimeError) as e:
            CloudPyCallback._create_client()
        assert str(e.value) == "Mocked error for testing"

        # 写入登录信息后不报错
        CloudPyCallback.login_info = login_info
        callback = CloudPyCallback()
        client = callback._create_client()
        assert client == get_client()
        assert CloudPyCallback.login_info is None


@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
def test_recreate_http(monkeypatch):
    """
    测试登录信息不存在时的行为
    1. 如果 CloudPyCallback.login_info 存在，则使用该登录信息创建客户端
    2. 如果环境变量中存在API_KEY，则使用该API_KEY登录
    3. 否则让进入交互，让用户输入API_KEY登录
    """
    api_key = os.getenv(SwanLabEnv.API_KEY.value)
    del os.environ[SwanLabEnv.API_KEY.value]

    # 输入 api-key 登录
    def input_api_key(_):
        return api_key

    def raise_error(_):
        raise RuntimeError("Should not be called")

    # 1. 如果 CloudPyCallback.login_info 存在，则使用该登录信息创建客户端
    monkeypatch.setattr("getpass.getpass", raise_error)
    login_info = login_by_key(api_key, save=False)
    with pytest.raises(RuntimeError) as e:
        CloudPyCallback._create_client()
    assert str(e.value) == "Should not be called"
    CloudPyCallback.login_info = login_info
    callback = CloudPyCallback()
    client = callback._create_client()
    assert CloudPyCallback.login_info is None
    assert client == get_client()
    callback.porter.close()
    reset_client()
    with pytest.raises(ValueError):
        get_client()
    # 2. 如果环境变量中存在API_KEY，则使用该API_KEY登录
    callback = CloudPyCallback()
    assert callback.login_info is None
    os.environ[SwanLabEnv.API_KEY.value] = api_key
    client2 = callback._create_client()
    assert client2 == get_client()
    assert client != client2
    callback.porter.close()
    reset_client()

    # 3. 否则让进入交互，让用户输入API_KEY登录
    monkeypatch.setattr("getpass.getpass", input_api_key)
    callback = CloudPyCallback()
    assert callback.login_info is None
    client3 = callback._create_client()
    assert client3 == get_client()
    assert client2 != client3 != client


@pytest.mark.skipif(T.is_skip_cloud_test, reason="skip cloud test")
class TestCloudResume:
    """
    测试云端恢复功能
    """

    @staticmethod
    def mock_run_state(workspace: str, project="test-cloud-resume"):
        run_store = get_run_store()
        run_store.project = project
        run_store.workspace = workspace
        run_store.visibility = True
        return run_store

    def test_init_config(self):
        """
        测试配置初始化
        """
        config = {
            "name": "test_cloud_resume",
            "description": "This is a test experiment for cloud resume.",
            "epochs": 10,
            "learning_rate": 0.001,
            "batch_size": 32,
        }
        run = swanlab.init(config=config, project="test-cloud-resume")
        run.finish()
        for key in run.config:
            assert run.config[key] == config[key]
        swanlab.login()
        client = get_client()
        # resume 实验
        with UseMockRunState(run_id=run.id, client=client):
            run_store = self.mock_run_state(client.username, run.public.project_name)
            run_store.resume = "must"
            callback = CloudPyCallback()
            callback.on_init()
            assert run_store.config is not None, "Config should be loaded when resuming."
            assert run_store.config == config, "Config should be loaded when resuming."



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/3 15:43
@File: pytest_line.py
@IDE: pycharm
@Description:
    测试折线图模块
"""
import math
import pytest
from swanlab.error import DataTypeError
from swanlab.data.modules import Line


def test_line_ok():
    """
    正常的Line解析
    """
    line = Line(1)
    data, raw = line.parse()
    assert data == 1
    assert raw is None
    assert line.get_chart() == line.Chart.LINE
    assert line.get_section() == "default"
    assert line.get_more() is None
    line = Line("1")
    data, raw = line.parse()
    assert data == 1
    assert raw is None

    class MockType:
        def __float__(self):
            return 1.

    line = Line(MockType())
    data, raw = line.parse()
    assert data == 1
    assert raw is None


def test_line_nan():
    """
    NaN解析
    """
    line = Line("NaN")
    data, raw = line.parse()
    assert data == Line.nan
    assert raw is None
    line = Line(math.nan)
    data, raw = line.parse()
    assert data == Line.nan
    assert raw is None


def test_line_inf():
    """
    INF解析
    """
    line = Line("INF")
    data, raw = line.parse()
    assert data == Line.inf
    assert raw is None
    line = Line(math.inf)
    data, raw = line.parse()
    assert data == Line.inf
    assert raw is None


def test_line_error():
    """
    错误解析
    """
    line = Line("a")
    with pytest.raises(DataTypeError) as e:
        line.parse()
    assert e.value.expected == "float"
    assert e.value.got == "str"
    line = Line([1])
    with pytest.raises(DataTypeError) as e:
        line.parse()
    assert e.value.expected == "float"
    assert e.value.got == "list"
    line = Line(None)
    with pytest.raises(DataTypeError) as e:
        line.parse()
    assert e.value.expected == "float"
    assert e.value.got == "NoneType"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-03-04 13:26:59
@File: test/create_experiment.py
@IDE: vscode
@Description:
    创建一个文件，作为测试用例
    为了不污染正式环境，在测试时需要切到测试/开发环境的配置文件
    测试后的swanlog文件存放于test/temp/swanlog目录下
    WARNING 请勿随意修改此文件，以免影响测试效果
"""
from tutils import open_dev_mode
import swanlab
import time
import random
import numpy as np

# 手动登录，所以要求config内写入的是测试环境的配置
swanlab.login(api_key=open_dev_mode())

epochs = 50
lr = 0.01
offset = random.random() / 5
# 初始化
swanlab.init(
    description="这是一个测试实验",
    log_level="debug",
    config="test/config/config.json",
    load="test/config/load.yaml",
    mode="cloud",
)
swanlab.config.epochs = epochs
swanlab.config.learning_rate = lr
swanlab.config.debug = "这是一串" + "很长" * 100 + "的字符串"
# 模拟训练
for epoch in range(2, swanlab.config.epochs):
    acc = 1 - 2**-epoch - random.random() / epoch - offset
    loss = 2**-epoch + random.random() / epoch + offset
    loss2 = 3**-epoch + random.random() / epoch + offset * 3
    print(f"epoch={epoch}, accuracy={acc}, loss={loss}")
    if epoch % 10 == 0:
        # 测试audio
        sample_rate = 44100
        audios = [swanlab.Audio(np.random.randn(2, 100000), sample_rate, caption="test") for _ in range(epoch // 10)]
        swanlab.log({"test/audio": audios}, step=epoch)
        # 测试image
        images = [swanlab.Image(np.random.randint(0, 255, (100, 100, 3)), caption="test") for _ in range(epoch // 10)]
        swanlab.log({"test/image": images}, step=epoch)
        # 测试text
        swanlab.log({"text": swanlab.Text("这是一段测试文本", caption="test")}, step=epoch)
        # 测试折线图
        swanlab.log({"t/accuracy": acc, "loss": loss, "loss2": loss2})
    else:
        # 测试折线图
        swanlab.log({"t/accuracy": acc, "loss": loss, "loss2": loss2})
    time.sleep(0.5)



"""
// @author: ComPleHN
// @file: line3d.vue
// @time: 2025/5/27 18:39
// @description: 本文件是对于echarts的 3D折线图 图表测试 
"""
# ---------------------------------------------- Line3d - Line3d_autorotate ----------------------------------------------
import math

from pyecharts import options as opts
from pyecharts.charts import Line3D
from pyecharts.faker import Faker

data = []
for t in range(0, 25000):
    _t = t / 1000
    x = (1 + 0.25 * math.cos(75 * _t)) * math.cos(_t)
    y = (1 + 0.25 * math.cos(75 * _t)) * math.sin(_t)
    z = _t + 2.0 * math.sin(75 * _t)
    data.append([x, y, z])
c1 = (
    Line3D()
    .add(
        "",
        data,
        xaxis3d_opts=opts.Axis3DOpts(Faker.clock, type_="value"),
        yaxis3d_opts=opts.Axis3DOpts(Faker.week_en, type_="value"),
        grid3d_opts=opts.Grid3DOpts(
            width=100, depth=100, rotate_speed=150, is_rotate=True
        ),
    )
    .set_global_opts(
        visualmap_opts=opts.VisualMapOpts(
            max_=30, min_=0, range_color=Faker.visual_color
        ),
        title_opts=opts.TitleOpts(title="Line3D-旋转的弹簧"),
    )
)

# ---------------------------------------------- Line3d - Line3d_rectangular_projection ----------------------------------------------
import math

import pyecharts.options as opts
from pyecharts.charts import Line3D

week_en = "Saturday Friday Thursday Wednesday Tuesday Monday Sunday".split()
clock = (
    "12a 1a 2a 3a 4a 5a 6a 7a 8a 9a 10a 11a 12p "
    "1p 2p 3p 4p 5p 6p 7p 8p 9p 10p 11p".split()
)

data = []
for t in range(0, 25000):
    _t = t / 1000
    x = (1 + 0.25 * math.cos(75 * _t)) * math.cos(_t)
    y = (1 + 0.25 * math.cos(75 * _t)) * math.sin(_t)
    z = _t + 2.0 * math.sin(75 * _t)
    data.append([x, y, z])

c2 = (
    Line3D()
    .add(
        "",
        data,
        xaxis3d_opts=opts.Axis3DOpts(data=clock, type_="value"),
        yaxis3d_opts=opts.Axis3DOpts(data=week_en, type_="value"),
        grid3d_opts=opts.Grid3DOpts(width=100, height=100, depth=100),
    )
    .set_global_opts(
        visualmap_opts=opts.VisualMapOpts(
            dimension=2,
            max_=30,
            min_=0,
            range_color=[
                "#313695",
                "#4575b4",
                "#74add1",
                "#abd9e9",
                "#e0f3f8",
                "#ffffbf",
                "#fee090",
                "#fdae61",
                "#f46d43",
                "#d73027",
                "#a50026",
            ],
        )
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="line3d",
    public=True,
)

swanlab.log(
    {
        "Line3d - Line3d_autorotate": c1,
        "Line3d - Line3d_rectangular_projection": c2
    }
)




"""
@author: cunyue
@file: v1.py
@time: 2025/6/20 15:16
@description: swanlab data transfer protocol version 1
基于protobuf的数据模型将自动实现数据从python到go的转换，由于我们的数据在最开始基于swanlab.toolkit中的模型定义，因此我们需要在这里实现从toolkit模型到protobuf模型的转换
仅需几个函数实现字段映射即可
"""



"""
@author: cunyue
@file: test_toolkit_logger.py
@time: 2025/10/10 22:27
@description: 测试工具中的日志模块
"""

import nanoid

from swanlab.toolkit import SwanKitLogger


class TestSwanKitLog:
    """
    测试日志模块
    """

    def test_enable_default(self, capsys):
        """
        测试开启日志
        """
        levels = ["debug", "info", "warning", "error", "critical"]
        for level in levels:
            name = nanoid.generate()
            text = nanoid.generate()
            t = SwanKitLogger(name, level=level)
            for le in levels:
                getattr(t, le)(text)
                out, err = capsys.readouterr()
                if levels.index(le) >= levels.index(level):
                    assert text in out
                    assert name in out
                    assert err == ""
                else:
                    assert out == ""
                    assert err == ""

    def test_disable(self, capsys):
        """
        测试关闭日志
        """
        levels = ["debug", "info", "warning", "error", "critical"]
        for level in levels:
            name = nanoid.generate()
            text = nanoid.generate()
            t = SwanKitLogger(name, level=level)
            t.disable_log()
            for le in levels:
                getattr(t, le)(text)
                out, err = capsys.readouterr()
                assert out == ""
                assert err == ""

    def test_set_level(self, capsys):
        """
        测试设置日志等级
        """
        levels = ["debug", "info", "warning", "error", "critical"]
        for le in levels:
            name = nanoid.generate()
            text = nanoid.generate()
            t = SwanKitLogger(name, level="debug")
            t.level = le
            getattr(t, le)(text)
            out, err = capsys.readouterr()
            assert text in out
            assert name in out
            assert err == ""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2023-12-15 17:33:56
@File: swanlab/log/__init__.py
@IDE: vscode
@Description:
    日志记录模块，在设计上swanlog作为一个独立的模块被使用
"""
from .log import SwanLog

swanlog: SwanLog = SwanLog("swanlab")

start_proxy = swanlog.start_proxy

reset = swanlog.reset

__all__ = ["start_proxy", "reset", "swanlog"]



from typing import List, Union

from swanlab.toolkit import BaseType, MediaBuffer, MediaType
from .audio import Audio
from .custom_charts import echarts, Echarts, PyEchartsBase, PyEchartsTable
from .image import Image
from .line import FloatConvertible, Line
from .object3d import Object3D, Molecule
from .text import Text
from .line import Line, FloatConvertible
from typing import Union, List
from .video import Video
from .wrapper import DataWrapper
from .custom_charts.metrics import roc_curve, pr_curve, confusion_matrix

DataType = Union[
    int,
    float,
    FloatConvertible,
    BaseType,
    List[BaseType],
    PyEchartsBase,
    PyEchartsTable,
    List[PyEchartsTable],
    List[PyEchartsBase],
]

ChartType = BaseType.Chart

__all__ = [
    # 数据类型
    "FloatConvertible",
    "DataWrapper",
    "MediaType",
    "PyEchartsBase",
    "PyEchartsTable",
    "DataType",
    "ChartType",
    "MediaBuffer",
    # 支持的图表类
    "Image",
    "Audio",
    "Text",
    "Line",
    "Object3D",
    "Molecule",
    "Echarts",
    "Video",
    # 模块
    "echarts",
    # metrics,
    "roc_curve",
    "pr_curve",
    "confusion_matrix",
]



import os
import swanlab
from datetime import datetime
from ._utils import find_tfevents, get_tf_events_tags_type, get_tf_events_tags_data
from swanlab.log import swanlog as swl
import time as t


class TFBConverter:
    def __init__(
        self,
        convert_dir: str,
        project: str = None,
        workspace: str = None,
        mode: str = "cloud",
        logdir: str = None,
        **kwargs,
    ):
        self.convert_dir = convert_dir
        self.project = project
        self.workspace = workspace
        self.mode = mode
        self.logdir = logdir

    def run(self, depth=3):
        swl.info("Start converting TFEvent files to SwanLab format...")
        timestamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")

        # 找到所有TFEvent文件, 生成一个路径字典
        path_dict = find_tfevents(self.convert_dir, depth=depth)
        if path_dict:
            swl.info("Found TFEvent file path dictionary.")
        else:
            swl.error(f"No TFEvent file found in {self.convert_dir}, please check the path.")
            return

        for dir, paths in path_dict.items():
            for path in paths:
                filename = os.path.basename(path)

                """
                获取所有的tag与其对应的类型, example:
                type_by_tags = {'training/loss': 'scalar', 'fake image': 'image', 'fake audio': 'audio', 'fake text/text_summary': 'text'}
                """
                type_by_tags = get_tf_events_tags_type(path)

                # 如果有tag（即该日志文件有记录指标，而非空文件）
                if type_by_tags:
                    # 初始化一个SwanLab实验
                    run = swanlab.init(
                        project=(f"Tensorboard-Conversion-{timestamp}" if self.project is None else self.project),
                        experiment_name=f"{dir}/{filename}",
                        workspace=self.workspace,
                        config={"tfevent_path": path},
                        mode=self.mode,
                        logdir=self.logdir,
                    )

                    """
                    根据tag提取数据, 格式为{tag: [(step, value, wall_time), ...]}, example:
                    data_by_tags = {
                        'training_loss': [
                            (0, 0.0, 1715839693),
                            (1, 0.019999999552965164, 1715839711),
                            (2, 0.03999999910593033, 1715839717)
                            ],
                        ...
                    }
                    """
                    data_by_tags = get_tf_events_tags_data(path, type_by_tags)

                    times = []
                    # 遍历数据
                    if data_by_tags:
                        # 打印并转换数据到SwanLab
                        index = 0
                        for tag, data in data_by_tags.items():
                            for step, value, time in data:
                                times.append(time)
                                # 如果是标量
                                if type_by_tags[tag] == "scalar":
                                    swanlab.log({tag: value}, step=step)
                                # 如果是图片
                                elif type_by_tags[tag] == "image":
                                    swanlab.log({tag: swanlab.Image(value)}, step=step)
                                # 如果是音频
                                elif type_by_tags[tag] == "audio":
                                    swanlab.log({tag: swanlab.Audio(value[0], sample_rate=value[1])}, step=step)
                                # 如果是文本
                                elif type_by_tags[tag] == "text":
                                    swanlab.log({tag: swanlab.Text(value)}, step=step)
                                # TODO: 随着SwanLab的发展，支持转换更多类型
                            # TODO: 等未来上传方案优化后解除延时
                            if index % 5 == 0:
                                t.sleep(1)
                            index += 1
                            print(f"Index {index}: Metric: {tag} log finished")

                    # 计算完整的运行时间
                    runtime = max(times) - min(times)
                    swanlab.config.update({"RunTime(s)": runtime})

                    # 结束当前实验
                    run.finish()



"""
@author: cunyue
@file: test_set_run_store.py
@time: 2025/7/21 17:19
@description: 测试 set_run_store 功能
"""

import nanoid
import pytest

from swanlab.proto.v0 import Project, Experiment
from swanlab.sync.sync_utils import set_run_store
from tutils.setup import UseMockRunState


@pytest.fixture
def temp_project() -> Project:
    """
    mock Project object for testing.
    """
    return Project.model_validate(
        {
            "name": "temp-project",
            "workspace": "temp-workspace",
            "public": False,
        }
    )


@pytest.fixture
def temp_experiment() -> Experiment:
    """
    mock Experiment object for testing.
    """
    return Experiment.model_validate(
        {
            "id": nanoid.generate(alphabet="0123456789abcdefghijklmnopqrstuvwxyz", size=21),
            "name": "temp-experiment",
            "colors": ("red", "blue"),
            "description": "Temp experiment for testing.",
            "tags": ["test", "sync"],
        }
    )


class TestSetRunStore:
    """
    测试设置运行存储的相关信息。
    """

    def test_default(self, temp_project, temp_experiment):
        with UseMockRunState() as mock:
            run_store = mock.store
            set_run_store(run_store, temp_project, temp_experiment)
            assert run_store.run_id is None, "Run ID should be None by default."
            assert run_store.resume == "never", "Resume should be set to 'never' by default."
            assert run_store.run_name == "temp-experiment", "Run name should match the experiment name."
            assert run_store.visibility is False, "Visibility should be False by default."
            assert run_store.workspace == "temp-workspace", "Workspace should match the project workspace."
            assert run_store.project == "temp-project", "Project should match the project project."
            assert run_store.run_colors == ("red", "blue"), "Run colors should match the experiment colors."
            assert run_store.tags == ["test", "sync"], "Run tags should match the experiment tags."
            assert (
                run_store.description == "Temp experiment for testing."
            ), "Run description should match the experiment description."

    def test_set_project(self, temp_project, temp_experiment):
        with UseMockRunState() as mock:
            run_store = mock.store
            set_run_store(run_store, temp_project, temp_experiment, project="custom-project")
            assert run_store.project == "custom-project", "Project should be set to 'custom-project'."

    def test_set_workspace(self, temp_project, temp_experiment):
        with UseMockRunState() as mock:
            run_store = mock.store
            set_run_store(run_store, temp_project, temp_experiment, workspace="custom-workspace")
            assert run_store.workspace == "custom-workspace", "Workspace should be set to 'custom-workspace'."

    def test_set_id_new(self, temp_project, temp_experiment):
        with UseMockRunState() as mock:
            run_store = mock.store
            set_run_store(run_store, temp_project, temp_experiment, id="new")
            assert run_store.run_id is None, "Run ID should be equal to the experiment ID."
            assert run_store.resume == "never", "Resume should be 'never' for 'new'."

    def test_set_id_auto(self, temp_project, temp_experiment):
        with UseMockRunState() as mock:
            run_store = mock.store
            set_run_store(run_store, temp_project, temp_experiment, id="auto")
            assert run_store.run_id == temp_experiment.id, "Run ID should match the experiment ID."
            assert run_store.resume == "allow", "Resume should be 'allow' for 'auto'."

    def test_set_id_custom(self, temp_project, temp_experiment):
        with UseMockRunState() as mock:
            run_store = mock.store
            custom_id = nanoid.generate(alphabet="0123456789abcdefghijklmnopqrstuvwxyz", size=21)
            set_run_store(run_store, temp_project, temp_experiment, id=custom_id)
            assert run_store.run_id == custom_id, "Run ID should be set to the custom ID."
            assert run_store.resume == "must", "Resume should be 'must' for a specific ID."



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/5/1 17:36
@File: base.py
@IDE: pycharm
@Description:
    SwanLab OpenAPI API基类
"""
import json
from datetime import datetime, timezone
from typing import Any, Callable, List, Optional, Union

import requests

from swanlab.api.types import ApiResponse
from swanlab.core_python import auth, create_session
from swanlab.log.log import SwanLog

_logger: Optional[SwanLog] = None

def get_logger(log_level: str = "info") -> SwanLog:
    global _logger
    if _logger is None:
        _logger = SwanLog("swanlab.openapi", log_level)
    else:
        _logger.level = log_level
    return _logger

def handle_response(resp: requests.Response) -> ApiResponse:
    try:
        data = resp.json() if resp.content else {}
    except (json.decoder.JSONDecodeError, requests.JSONDecodeError):
        return ApiResponse[str](
            code=resp.status_code,
            errmsg="sdk decode json error",
            data=resp.text
        )

    if not isinstance(data, (dict, list)):
        return ApiResponse[Any](
            code=resp.status_code,
            errmsg="sdk decode dict error",
            data=data
        )

    code = resp.status_code
    if 200 <= code < 300:
        message = ""
    else:
        message = f"api error: {resp.reason}. Trace id: {resp.headers.get('traceid')}"
    return ApiResponse(
        code=code,
        errmsg=message,
        data=data
    )


class ApiHTTP:
    REFRESH_TIME = 60 * 60 * 24 * 7  # 7天

    def __init__(self, login_info: auth.LoginInfo):
        self.__logger = get_logger()
        self.__login_info: auth.LoginInfo = login_info
        self.__session: requests.Session = self.__init_session()
        self.service: OpenApiService = OpenApiService(self)
    
    @property
    def session(self) -> requests.Session:
        """
        获取当前的requests.Session对象
        """
        return self.__session

    @property
    def username(self) -> str:
        """
        当前登录的用户名
        """
        return self.__login_info.username or ""

    @property
    def base_url(self):
        return self.__login_info.api_host

    @property
    def sid_expired_at(self):
        """
        获取sid的过期时间
        """
        return datetime.strptime(self.__login_info.expired_at or "", "%Y-%m-%dT%H:%M:%S.%fZ")

    def __init_session(self) -> requests.Session:
        session = create_session()
        session.cookies.update({"sid": self.__login_info.sid or ""})
        return session

    def __before_request(self):
        if (self.sid_expired_at - datetime.now(timezone.utc).replace(tzinfo=None)).total_seconds() < self.REFRESH_TIME:
            self.__logger.debug("Refreshing sid...")
            self.__login_info = auth.login_by_key(self.__login_info.api_key or "", save=False)
            self.__session.headers["cookie"] = f"sid={self.__login_info.sid}"

    def get(self, url: str, params: dict) -> ApiResponse:
        self.__before_request()
        resp = self.__session.get(self.base_url + url, params=params)
        return handle_response(resp)

    def post(self, url: str, data: Union[dict, list], params: dict) -> ApiResponse:
        self.__before_request()
        resp = self.__session.post(self.base_url + url, json=data, params=params)
        return handle_response(resp)

    def delete(self, url: str, params: dict) -> ApiResponse:
        self.__before_request()
        resp = self.__session.delete(self.base_url + url, params=params)
        return handle_response(resp)

class OpenApiService:
    def __init__(self, http: ApiHTTP):
        self.http: ApiHTTP = http

    def get_exp_info(self, username: str, project: str, exp_id: str) -> ApiResponse[dict]:
        """
        获取实验信息
        """
        return self.http.get(f"/project/{username}/{project}/runs/{exp_id}", params={})

    def get_project_info(self, username: str, projname: str) -> ApiResponse[dict]:
        """
        获取项目详情
        """
        return self.http.get(f"/project/{username}/{projname}", params={})
    
    @staticmethod
    def fetch_paginated_api(
        api_func: Callable[..., ApiResponse],  # 分页 API 请求函数
        page_field: str = "page",
        size_field: str = "size",
        page_size: int = 10,
        *args, **kwargs
    ) -> ApiResponse[List]:
        """
        通用分页全量拉取函数

        Args:
            api_func (Callable): 分页 API 请求函数，应返回 ApiResponse[Pagination]
            page_field (str): 页码参数名，默认为 "page"
            size_field (str): 每页大小参数名，默认为 "size"
            page_size (int): 每页条数，默认为 10
            *args: 传递给 api_func 的位置参数
            **kwargs: 传递给 api_func 的关键字参数

        Returns:
            ApiResponse[list]: 返回所有分页数据组成的 ApiResponse
        """
        page = 1
        objs = []
        while True:
            kwargs.update({page_field: page, size_field: page_size})
            resp: ApiResponse = api_func(*args, **kwargs)
            if resp.errmsg:
                break

            objs.extend(resp.data.list)

            if len(objs) >= resp.data.total:
                break
            page += 1

        return ApiResponse[List](code=resp.code, errmsg=resp.errmsg, data=objs)

class ApiBase:
    def __init__(self, http: ApiHTTP):
        self.http: ApiHTTP = http



"""
@author: cunyue
@file: sync.py
@time: 2025/7/20 17:23
@description: 测试同步功能，与同目录下的sync.sh文件配合使用，使用方式为：
1. python test/sync/sync.py: 先运行 sync.sh，此时会要求输入 run_dir
2. bash ./test/sync/sync.s: 然后在项目根目录下运行此脚本，他会打印出当前实验的路径，复制此路径到步骤一，按回车
这样将实现每五秒钟同步一次数据到云端
"""

import os
import random
import time

import numpy as np

import swanlab
from tutils import TEMP_PATH

logdir = os.path.join(TEMP_PATH, "sync")

run = swanlab.init(logdir=logdir, mode="offline", project="test-sync", description="test-sync", experiment_name="sync")

input("日志文件夹路径为: " + run.public.run_dir + " , 按任意键进入下一步")

epochs = 500
lr = 0.01
offset = random.random() / 5

swanlab.config.epochs = epochs
swanlab.config.learning_rate = lr
swanlab.config.offset = offset

# 模拟训练
for epoch in range(2, swanlab.config.epochs):
    acc = 1 - 2**-epoch - random.random() / epoch - offset
    loss = 2**-epoch + random.random() / epoch + offset
    print(f"epoch={epoch}, accuracy={acc}, loss={loss}")
    if epoch % 10 == 0:
        # 测试image
        images = [swanlab.Image(np.random.randint(0, 255, (100, 100, 3)), caption="test") for _ in range(epoch // 10)]
        swanlab.log({"test/image": images}, step=epoch)
        # 测试折线图
        swanlab.log({"t/accuracy": acc, "loss": loss}, step=epoch)
    else:
        # 测试折线图
        swanlab.log({"t/accuracy": acc, "loss": loss}, step=epoch)
    time.sleep(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2023-11-30 01:39:04
@File: swanlab\cli\main.py
@IDE: vscode
@Description:
    swanlab脚本命令的主入口
"""
import click

import swanlab.cli.commands as C
from swanlab.package import get_package_version


@click.group(invoke_without_command=True)
@click.version_option(get_package_version(), "--version", "-v", message="SwanLab %(version)s")
def cli():
    pass


# ---------------------------------- 添加其他命令 ----------------------------------


# noinspection PyTypeChecker
cli.add_command(C.login)  # 登录
# noinspection PyTypeChecker
cli.add_command(C.logout)  # 登出
# noinspection PyTypeChecker
cli.add_command(C.verify)  # 验证当前登录状态

# noinspection PyTypeChecker
cli.add_command(C.watch)  # 启动服务

# noinspection PyTypeChecker
cli.add_command(C.convert)  # 转换命令，用于转换其他实验跟踪工具

# noinspection PyTypeChecker
cli.add_command(C.sync)  # 同步命令，用于同步本地数据到云端

# noinspection PyTypeChecker
cli.add_command(C.offline)  # 开启默认离线模式

# noinspection PyTypeChecker
cli.add_command(C.local)  # 开启默认本地模式

# noinspection PyTypeChecker
cli.add_command(C.online)  # 开启默认在线模式

# noinspection PyTypeChecker
cli.add_command(C.disabled)  # 开启默认禁用模式


if __name__ == "__main__":
    cli()



"""
@author: cunyue
@file: upload.py
@time: 2025/6/16 14:13
@description: 定义上传函数
"""

from typing import List, Union, Literal, TypedDict

from swanlab.log import swanlog
from .model import ColumnModel, MediaModel, ScalarModel, FileModel, LogModel
from ..client import get_client, sync_error_handler, decode_response
from ...error import ApiError

house_url = '/house/metrics'


# 上传指标数据
class MetricDict(TypedDict):
    projectId: str
    experimentId: str
    type: str
    metrics: List[dict]
    flagId: Union[str, None]


def create_data(metrics: List[dict], metrics_type: str) -> MetricDict:
    """
    携带上传日志的指标信息
    """
    client = get_client()
    # Move 等实验需要将数据上传到根实验上
    exp_id = client.exp.root_exp_cuid or client.exp.cuid
    proj_id = client.exp.root_proj_cuid or client.proj.cuid
    assert proj_id is not None, "Project ID is empty."
    assert exp_id is not None, "Experiment ID is empty."
    flag_id = client.exp.flag_id
    return {
        "projectId": proj_id,
        "experimentId": exp_id,
        "type": metrics_type,
        "metrics": metrics,
        "flagId": flag_id,
    }


def trace_metrics(
    url: str,
    data: Union[MetricDict, list] = None,
    method: Literal['post', 'put'] = 'post',
    per_request_len: int = 1000,
):
    """
    创建指标数据方法，如果 client 处于挂起状态，则不进行上传
    :param url: 上传的URL地址
    :param data: 上传的数据，可以是字典或列表
    :param method: 请求方法，默认为 'post'
    :param per_request_len: 每次请求的最大数据长度，如果设置为-1则不进行分批上传
    """
    # TODO 用装饰器设置client的pending状态
    client = get_client()
    if client.pending:
        return
    if per_request_len == -1:
        _, resp = getattr(client, method)(url, data)
        if resp.status_code == 202:
            client.pending = True
            return
        return
    # 分批上传
    if isinstance(data, dict):
        # 1. 指标数据
        for i in range(0, len(data['metrics']), per_request_len):
            _, resp = getattr(client, method)(
                url,
                {
                    **data,
                    "metrics": data['metrics'][i : i + per_request_len],
                },
            )
            if resp.status_code == 202:
                client.pending = True
                return
    else:
        # 2. 列表数据（列等）
        for i in range(0, len(data), per_request_len):
            _, resp = getattr(client, method)(url, data[i : i + per_request_len])
            if resp.status_code == 202:
                client.pending = True
                return


@sync_error_handler
def upload_logs(logs: List[LogModel]):
    """
    上传日志信息
    :param logs: 日志信息集合
    """
    metrics = []
    for log in logs:
        metrics.extend([{"level": log['level'], **l} for l in log['contents']])
    if len(metrics) == 0:
        return swanlog.debug("No logs to upload.")
    data = create_data(metrics, "log")
    trace_metrics(house_url, data)
    return None


@sync_error_handler
def upload_media_metrics(media_metrics: List[MediaModel]):
    """
    上传指标的媒体数据
    :param media_metrics: 媒体指标数据集合
    """
    client = get_client()
    buffers = []
    for media in media_metrics:
        media.buffers and buffers.extend(media.buffers)
    if not client.pending:
        client.upload_files(buffers)
        # 上传指标信息
        trace_metrics(house_url, create_data([x.to_dict() for x in media_metrics], MediaModel.type.value))


@sync_error_handler
def upload_scalar_metrics(scalar_metrics: List[ScalarModel]):
    """
    上传指标的标量数据
    """
    data = create_data([x.to_dict() for x in scalar_metrics], ScalarModel.type.value)
    trace_metrics(house_url, data)


@sync_error_handler
def upload_files(files: List[FileModel]):
    """
    上传files文件夹中的内容
    :param files: 文件列表，内部为文件信息（text/dict）
    """
    http = get_client()
    # 去重所有的FileModel，留下一个
    if len(files) == 0:
        swanlog.warning("No files to upload.")
        return
    file_model = FileModel.create(files)
    # 如果没有文件需要上传，直接返回
    if file_model.empty:
        return
    data = file_model.to_dict()
    trace_metrics(
        f'/project/{http.groupname}/{http.projname}/runs/{http.exp_id}/profile',
        data,
        method="put",
        per_request_len=-1,
    )
    return


@sync_error_handler
def upload_columns(columns: List[ColumnModel]):
    """
    批量上传并创建 columns，每个请求的列长度有一个最大值
    """
    http = get_client()
    url = f'/experiment/{http.exp_id}/columns'
    # 如果列表长度为0，则跳过
    if len(columns) == 0:
        swanlog.debug("No columns to upload.")
        return
    # 分批上传
    try:
        trace_metrics(url, [x.to_dict() for x in columns], per_request_len=3000)
    except ApiError as e:
        # 处理实验不存在的异常
        if e.resp.status_code == 404:
            resp = decode_response(e.resp)
            # 实验不存在，那么剩下的列也没有必要上传了，直接返回
            if isinstance(resp, dict) and resp.get('code') == 'Disabled_Resource':
                swanlog.warning(f"Experiment {http.exp_id} has been deleted, skipping column upload.")
                return
        raise e


__all__ = [
    "upload_logs",
    "upload_media_metrics",
    "upload_scalar_metrics",
    "upload_files",
    "upload_columns",
]



"""
@author: zeyi-lin
@file: kunlunxin.py
@time: 2025/04/14 12:32
@description: 昆仑芯xpu信息采集
"""

import math
import platform
import subprocess
from typing import Tuple, Optional

from ..type import HardwareFuncResult, HardwareInfoList, HardwareConfig, HardwareCollector as H
from ..utils import generate_key, random_index


def get_kunlunxin_xpu_info() -> HardwareFuncResult:
    """
    获取昆仑芯xpu信息，包括驱动版本、设备信息等
    """
    # kunlunxin芯片只支持Linux系统
    if platform.system() != "Linux":
        return None, None

    info = {"driver": None, "xpu": None}
    collector = None
    try:
        driver, xpu_map = map_xpu()
        info["driver"] = driver
        info["xpu"] = xpu_map
        collector = KunlunxinCollector(xpu_map)
    except Exception:  # noqa
        if all(v is None for v in info.values()):
            return None, None
    return info, collector


def get_xpu_driver() -> Tuple[Optional[str], dict]:
    """
    获取xpu驱动版本
    """
    output = subprocess.run(["xpu-smi", "-q"], capture_output=True, check=True, text=True).stdout
    lines = output.split("\n")
    for line in lines:
        if "Driver Version" in line:
            return line.split(":")[-1].strip()
    return None


def map_xpu() -> Tuple[Optional[str], dict]:
    """
    列出所有xpu设备，并包含芯片的映射关系
    返回值样例:
        driver: "1.1.0"
        xpu_map: {"0": { "name": "kunlunxin-910", "memory": 16, }, "1": { "name": "kunlunxin-910", "memory": 16, }, ...}
    """
    output = subprocess.run(["xpu-smi", "-m"], capture_output=True, check=True, text=True).stdout
    
    xpu_map = {}
    driver = get_xpu_driver()
    lines = output.split("\n")

    for line in lines:
        # 将line按空格分为列表
        try:
            xpu_map[line.split()[1]] = {
                "name": (line.split()[21][1:] + " " + line.split()[22][:-1]).strip(),
                "memory": int(line.split()[18])//1024,
            }
        except Exception:  # noqa
            continue
        
    return driver, xpu_map


class KunlunxinCollector(H):
    def __init__(self, xpu_map):
        super().__init__()
        self.xpu_map = xpu_map

        # xpu Utilization (%)
        self.util_key = generate_key("xpu.{xpu_index}.ptc")
        util_config = HardwareConfig(
            y_range=(0, 100),
            chart_index=random_index(),
            chart_name="XPU Utilization (%)",
        )
        self.per_util_configs = {}

        # xpu Memory Allocated (%)
        self.memory_key = generate_key("xpu.{xpu_index}.mem.ptc")
        memory_config = HardwareConfig(
            y_range=(0, 100),
            chart_index=random_index(),
            chart_name="XPU Memory Allocated (%)",
        )
        self.per_memory_configs = {}

        # xpu Temperature (°C)
        self.temp_key = generate_key("xpu.{xpu_index}.temp")
        temp_config = HardwareConfig(
            y_range=(0, None),
            chart_index=random_index(),
            chart_name="XPU Temperature (°C)",
        )
        self.per_temp_configs = {}

        # xpu Power (W)
        self.power_key = generate_key("xpu.{xpu_index}.power")
        power_config = HardwareConfig(
            y_range=(0, None),
            chart_index=random_index(),
            chart_name="XPU Power (W)",
        )
        self.per_power_configs = {}

        for xpu_id in self.xpu_map:
            metric_name = f"XPU {xpu_id}"
            self.per_util_configs[metric_name] = util_config.clone(metric_name=metric_name)
            self.per_memory_configs[metric_name] = memory_config.clone(metric_name=metric_name)
            self.per_temp_configs[metric_name] = temp_config.clone(metric_name=metric_name)
            self.per_power_configs[metric_name] = power_config.clone(metric_name=metric_name)

    def collect(self) -> HardwareInfoList:
        result: HardwareInfoList = []
        usage_methods = [
            self.get_utilization_usage,
            self.get_memory_usage,
            self.get_temperature_usage,
            self.get_power_usage,
        ]

        for method in usage_methods:
            result.extend(method().values())
        return result

    def get_utilization_usage(self) -> dict:
        """
        获取指定xpu设备的利用率
        """
        output = subprocess.run(["xpu-smi", "-m"], capture_output=True, text=True).stdout

        util_infos = {}
        xpu_ids = []

        # 获取所有xpu ID
        for xpu_id in self.xpu_map:
            xpu_ids.append(xpu_id)

        index = 0
        lines = output.split("\n")
        for line in lines:
            try: 
                
                util = line.split()[19]
                if util.isdigit():
                    util_infos[xpu_ids[index]] = {
                        "key": self.util_key.format(xpu_index=xpu_ids[index]),
                        "name": f"XPU {xpu_ids[index]} Utilization (%)",
                        "value": math.nan,
                        "config": self.per_util_configs[f"XPU {xpu_ids[index]}"],
                    }
                    util_infos[xpu_ids[index]]['value'] = float(util)
                    index += 1
            except Exception:  # noqa
                continue

        return util_infos

    def get_memory_usage(self) -> dict:
        """
        获取指定xpu设备的显存占用率
        """
        output = subprocess.run(["xpu-smi", "-m"], capture_output=True, text=True).stdout

        memory_infos = {}
        xpu_ids = []

        # 获取所有xpu ID
        for xpu_id in self.xpu_map:
            xpu_ids.append(xpu_id)

        index = 0
        lines = output.split("\n")
        for line in lines:
            try:
                memory = line.split()[17]
                if memory.isdigit():
                    memory_infos[xpu_ids[index]] = {
                        "key": self.memory_key.format(xpu_index=xpu_ids[index]),
                        "name": f"XPU {xpu_ids[index]} Memory Allocated (%)",
                        "value": math.nan,
                        "config": self.per_memory_configs[f"XPU {xpu_ids[index]}"],
                    }
                    memory_infos[xpu_ids[index]]['value'] = float(memory) / (self.xpu_map[xpu_ids[index]]['memory'] * 1024) * 100
                    index += 1
            except Exception:  # noqa
                continue

        return memory_infos

    def get_temperature_usage(self) -> dict:
        """
        获取指定xpu设备的温度(°C)
        """
        output = subprocess.run(["xpu-smi", "-m"], capture_output=True, text=True).stdout
        temp_infos = {}
        xpu_ids = []

        # 获取所有xpu ID
        for xpu_id in self.xpu_map:
            xpu_ids.append(xpu_id)

        index = 0
        lines = output.split("\n")
        for line in lines:
            try:
                temp = line.split()[4]
                if temp.isdigit():
                    temp_infos[xpu_ids[index]] = {
                        "key": self.temp_key.format(xpu_index=xpu_ids[index]),
                        "name": f"XPU {xpu_ids[index]} Temperature (°C)",
                        "value": math.nan,
                        "config": self.per_temp_configs[f"XPU {xpu_ids[index]}"],
                    }
                    temp_infos[xpu_ids[index]]['value'] = float(temp)
                    index += 1
            except Exception:  # noqa
                continue

        return temp_infos

    def get_power_usage(self) -> dict:
        """
        获取指定xpu设备的功耗(W)
        """
        output = subprocess.run(["xpu-smi", "-m"], capture_output=True, text=True).stdout
        power_infos = {}
        xpu_ids = []

        # 获取所有xpu ID
        for xpu_id in self.xpu_map:
            xpu_ids.append(xpu_id)

        index = 0
        lines = output.split("\n")
        for line in lines:
            try:
                power = line.split()[8]
                if power.isdigit():
                    power_infos[xpu_ids[index]] = {
                        "key": self.power_key.format(xpu_index=xpu_ids[index]),
                        "name": f"XPU {xpu_ids[index]} Power (W)",
                        "value": math.nan,
                        "config": self.per_power_configs[f"XPU {xpu_ids[index]}"],
                    }
                    power_infos[xpu_ids[index]]['value'] = float(power)
                    index += 1
            except Exception:  # noqa
                continue

        return power_infos



import swanlab

def _extract_args(args, kwargs, param_names):
    """
    从args和kwargs中提取参数值的通用函数
    
    Args:
        args: 位置参数元组
        kwargs: 关键字参数字典
        param_names: 参数名称列表
    
    Returns:
        tuple: 按param_names顺序返回提取的参数值
    """
    values = []
    for i, name in enumerate(param_names):
        if len(args) > i:
            values.append(args[i])
        else:
            values.append(kwargs.get(name, None))
    return tuple(values)


def sync_wandb(
    mode:str="cloud",
    wandb_run:bool=True,
    workspace:str=None,
    logdir:str=None,
    ):
    """
    sync wandb with swanlab, 暂时不支持log非标量类型
    
    - mode: "cloud", "local" or "disabled". https://docs.swanlab.cn/api/py-init.html
    - wandb_run: 如果此参数设置为False，则不会将数据上传到wandb，等同于设置wandb.init(mode="offline")。
    - workspace: swanlab的组织空间username
    - logdir: swanlab日志文件存储目录
    
    usecase:
    ```python
    import wandb
    import random
    import swanlab

    swanlab.sync_wandb()
    # swanlab.init(project="sync_wandb")

    wandb.init(
        project="test",
        config={"a": 1, "b": 2},
        name="test",
    )

    epochs = 10
    offset = random.random() / 5
    for epoch in range(2, epochs):
        acc = 1 - 2 ** -epoch - random.random() / epoch - offset
        loss = 2 ** -epoch + random.random() / epoch + offset

        wandb.log({"acc": acc, "loss": loss})
    ```
    """
    try:
        import wandb
        from wandb import sdk as wandb_sdk
    except ImportError:
        raise ImportError("please install wandb first, command: `pip install wandb`")
    
    original_init = wandb.init
    original_log = wandb_sdk.wandb_run.Run.log
    original_finish = wandb_sdk.wandb_run.Run.finish
    original_config_update = wandb_sdk.wandb_config.Config.update
    
    def patched_init(*args, **kwargs):
        entity, project, dir, id, name, notes, tags, config, config_exclude_keys, reinit = _extract_args(
            args, kwargs, ['entity', 'project', 'dir', 'id', 'name', 'notes', 'tags', 'config', 'config_exclude_keys', 'reinit']
        )
        
        if swanlab.data.get_run() is None:
            swanlab.init(
                project=project,
                workspace=workspace,
                experiment_name=name,
                description=notes,
                config=config,
                tags=tags,
                mode=mode,
                logdir=logdir,
                reinit=reinit,
                )
        else:
            swanlab.config.update(config)
        
        if wandb_run is False:
            kwargs["mode"] = "offline"
            return original_init(*args, **kwargs)
        else:
            return original_init(*args, **kwargs)

    def patched_config_update(self, *args, **kwargs):
        d, _ = _extract_args(args, kwargs, ['d', 'allow_val_change'])
        
        if d is not None:
            swanlab.config.update(d)
        return original_config_update(self, *args, **kwargs)

    def patched_log(self, *args, **kwargs):
        data, step, commit, sync = _extract_args(args, kwargs, ['data', 'step', 'commit', 'sync'])
        
        if data is None:
            return original_log(self, *args, **kwargs)
        
        # 处理数据，支持 wandb.Image
        processed_data = {}
        for key, value in data.items():
            if isinstance(value, (int, float, bool, str)):
                # 标量类型直接保留
                processed_data[key] = value
            elif hasattr(value, '__class__') and value.__class__.__name__ == 'Image' and hasattr(value, 'image'):
                # 检测是否为 wandb.Image
                try:
                    # 获取 wandb.Image 的图像数据
                    if value.image is not None:
                        # 将 PIL Image 转换为 numpy 数组
                        import numpy as np
                        img_array = np.array(value.image)
                        
                        # 创建 swanlab.Image
                        caption = getattr(value, '_caption', None)
                        swanlab_image = swanlab.Image(img_array, caption=caption)
                        processed_data[key] = swanlab_image
                    else:
                        # 如果 image 为 None，尝试使用 _image
                        if hasattr(value, '_image') and value._image is not None:
                            import numpy as np
                            img_array = np.array(value._image)
                            caption = getattr(value, '_caption', None)
                            swanlab_image = swanlab.Image(img_array, caption=caption)
                            processed_data[key] = swanlab_image
                except Exception as e:
                    # 如果转换失败，记录错误但继续处理其他数据
                    print(f"Warning: Failed to convert wandb.Image for key '{key}': {e}")
                    continue
            elif isinstance(value, list) and value and hasattr(value[0], '__class__') and value[0].__class__.__name__ == 'Image':
                # 检测是否为 wandb.Image 列表
                try:
                    import numpy as np
                    swanlab_images = []
                    for v in value:
                        if hasattr(v, 'image') and v.image is not None:
                            img_array = np.array(v.image)
                            caption = getattr(v, '_caption', None)
                            swanlab_images.append(swanlab.Image(img_array, caption=caption))
                        elif hasattr(v, '_image') and v._image is not None:
                            img_array = np.array(v._image)
                            caption = getattr(v, '_caption', None)
                            swanlab_images.append(swanlab.Image(img_array, caption=caption))
                    if swanlab_images:
                        processed_data[key] = swanlab_images
                except Exception as e:
                    # 如果转换失败，记录错误但继续处理其他数据
                    print(f"Warning: Failed to convert wandb.Image list for key '{key}': {e}")
                    continue
        
        if processed_data:
            swanlab.log(data=processed_data, step=step)
        
        return original_log(self, *args, **kwargs)
    
    def patched_finish(*args, **kwargs):
        swanlab.finish()
        return original_finish(*args, **kwargs)

    wandb.init = patched_init
    wandb_sdk.wandb_run.Run.log = patched_log
    wandb_sdk.wandb_run.Run.finish = patched_finish
    wandb_sdk.wandb_config.Config.update = patched_config_update


#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/12 21:11
@File: login.py
@IDE: pycharm
@Description:
    登录模块
"""
import os

import click
from rich.text import Text

from swanlab.core_python import auth
from swanlab.env import SwanLabEnv
from swanlab.log import swanlog
from swanlab.package import has_api_key, HostFormatter


@click.command()
@click.option(
    "--relogin",
    "-r",
    is_flag=True,
    default=False,
    help="Relogin to the swanlab cloud, it will recover the token file.",
)
@click.option(
    "--api-key",
    "-k",
    default=None,
    type=str,
    help="If you prefer not to engage in commands-line interaction to input the api key, "
    "this will allow automatic login.",
)
@click.option(
    "--host",
    "-h",
    default=None,
    type=str,
    help="The host of the swanlab server.",
)
@click.option(
    "--web-host",
    "-w",
    default=None,
    type=str,
    help="The web host of the swanlab cloud front-end.",
)
def login(api_key: str, relogin: bool, host: str, web_host: str):
    """Login to the swanlab cloud."""
    if not relogin and has_api_key():
        # 此时代表token已经获取，需要打印一条信息：已经登录
        return swanlog.info(
            "You are already logged in. Use `",
            Text("swanlab login --relogin", "bold"),
            "` to force relogin.",
            sep='',
        )
    # 清除环境变量
    if relogin:
        del os.environ[SwanLabEnv.API_HOST.value]
        del os.environ[SwanLabEnv.WEB_HOST.value]
    try:
        HostFormatter(host, web_host)()
    except ValueError as e:
        raise click.BadParameter(str(e))
    # 进行登录，此时将直接覆盖本地token文件
    login_info = auth.terminal_login(api_key)
    swanlog.info("Login successfully. Hi, ", Text(login_info.username, "bold"), "!", sep='')



try:
    import pytorch_lightning as pl
    from pytorch_lightning import demos # Need Import Again
except ImportError:
    pl = None

import swanlab
from swanlab.data.run import SwanLabRunState
from swanlab.integration.pytorch_lightning import SwanLabLogger


def pl_strategy_for_swanlab(raw_fn, swanlab_logger: SwanLabLogger):
    def wrapper(exception: BaseException):
        if isinstance(exception, KeyboardInterrupt):
            swanlab_logger.experiment.finish(SwanLabRunState.CRASHED, "KeyboardInterrupt by user", interrupt=True)
        raw_fn(exception)
    return wrapper


def test_for_pytorch_lighting():
    # Also See SwanLab/test/integration/lightning
    if pl is None:
        Warning("Need PyTorch Lightning To Test")
        return

    model = pl.demos.boring_classes.BoringModel()
    datamodule = pl.demos.boring_classes.BoringDataModule()
    logger = SwanLabLogger(project="test-interrupt")

    # log_every_n_steps != None for enable Logger
    trainer = pl.Trainer(max_epochs=10, logger=logger, log_every_n_steps=1)

    # 加上可使pl被KeyboardInterrupt中断时，Web上的状态为 "Ctrl+C" 而不是 "中断"
    # Add support so that when pl is interrupted by a KeyboardInterrupt, the status displayed on the web is "Ctrl+C" instead of "Crashed"
    # trainer.strategy.on_exception = pl_strategy_for_swanlab(trainer.strategy.on_exception, logger) # 在pl的策略补充即可

    trainer.fit(model, datamodule)

def test_for_normal_interrupt():
    from tqdm import tqdm
    # import time

    swanlab.init(project="test-interrupt")

    for i in tqdm(range(10000)):
        # time.sleep(0.01)

        for _ in range(100):
            for _ in range(100):
                for _ in range(100):
                    pass

        if i % 10 == 0:
            swanlab.log({"progress": i}, print_to_console=True)


if __name__ == '__main__':
    test_for_pytorch_lighting()

    # test_for_normal_interrupt()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/3 02:35
@File: __init__.py
@IDE: pycharm
@Description:
    文本模块
"""
from typing import Union

from swanlab.toolkit import MediaType, DataSuite as D


class Text(MediaType):
    """Text class constructor

    Parameters
    ----------
    data: str, float, int
        text data.
    caption: str
        caption for the data, it will be displayed in the dashboard.
        e.g. swanlab.Text("Hello World", caption="This is a caption for the text data.")
    """

    def __init__(self, data: Union[str, int, float], caption: str = None):
        super().__init__()
        # 处理文本数据

        if isinstance(data, str):
            self.text_data = data
        elif isinstance(data, (int, float)):
            self.text_data = str(data)
        else:
            raise TypeError("data must be a string, int or float.")

        # 处理caption
        self.caption = D.check_caption(caption)

    # ---------------------------------- 覆写 ----------------------------------

    def parse(self):
        return self.text_data, None

    def get_more(self, *args, **kwargs) -> dict:
        """返回config数据"""
        return {"caption": self.caption} if self.caption is not None else None

    def get_section(self, *args, **kwargs) -> str:
        """设定分组名"""
        return "Text"

    def get_chart(self):
        """设定图表类型"""
        return self.Chart.TEXT



import lightgbm as lgb
from sklearn.datasets import load_breast_cancer
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score
import swanlab
from swanlab.integration.lightgbm import SwanLabCallback

# Step 1: Initialize swanlab
swanlab.init(project="lightgbm-example", name="breast-cancer-classification")

# Step 2: Load the dataset
data = load_breast_cancer()
X = data.data
y = data.target

# Step 3: Split the data
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

# Step 4: Create LightGBM datasets
train_data = lgb.Dataset(X_train, label=y_train)
test_data = lgb.Dataset(X_test, label=y_test, reference=train_data)

# Step 5: Set parameters
params = {
    'objective': 'binary',
    'metric': 'binary_logloss',
    'boosting_type': 'gbdt',
    'num_leaves': 31,
    'learning_rate': 0.05,
    'feature_fraction': 0.9
}

# Step 6: Train the model with swanlab callback
num_round = 100
gbm = lgb.train(
    params,
    train_data,
    num_round,
    valid_sets=[test_data],
    callbacks=[SwanLabCallback()]
)

# Step 8: Make predictions
y_pred = gbm.predict(X_test, num_iteration=gbm.best_iteration)
y_pred_binary = [1 if p >= 0.5 else 0 for p in y_pred]

# Step 9: Evaluate the model
accuracy = accuracy_score(y_test, y_pred_binary)
print(f"模型准确率: {accuracy:.4f}")
swanlab.log({"accuracy": accuracy})

# Step 10: Save the model locally
gbm.save_model('lightgbm_model.txt')

# Step 11: Load the model and predict again
bst_loaded = lgb.Booster(model_file='lightgbm_model.txt')
y_pred_loaded = bst_loaded.predict(X_test)
y_pred_binary_loaded = [1 if p >= 0.5 else 0 for p in y_pred_loaded]

# Step 12: Evaluate the loaded model
accuracy_loaded = accuracy_score(y_test, y_pred_binary_loaded)
print(f"加载模型后的准确率: {accuracy_loaded:.4f}")
swanlab.log({"accuracy_loaded": accuracy_loaded})

# Step 13: Finish the swanlab run
swanlab.finish()